@@ -8,6 +8,9 @@
 // - setoption USI_Hash を無視 (常に最大を利用)
 
 #include "osl/record/usi.h"
+#include "osl/record/opening/openingBook.h"
+#include "osl/record/csaRecord.h"
+#include "osl/record/csaIOError.h"
 #include "osl/state/numEffectState.h"
 #include "osl/game_playing/speculativeSearchPlayer.h"
 #include "osl/game_playing/alphaBetaPlayer.h"
@@ -20,16 +23,16 @@
 #include "osl/eval/ml/openMidEndingEval.h"
 #include "osl/eval/progressEval.h"
 #include "osl/rating/featureSet.h"
-#include "osl/record/compactBoard.h"
 #include "osl/misc/pointerQueue.h"
 #include "osl/misc/ctime.h"
 #include "osl/misc/ncores.h"
 #include "osl/progress/ml/newProgress.h"
-#include "osl/record/opening/openingBook.h"
 #include "osl/move_generator/legalMoves.h"
+#include "osl/move_probability/featureSet.h"
 #include "osl/rating/ratingEnv.h"
 #include "osl/oslConfig.h"
 #include "osl/sennichite.h"
+#include "osl/enter_king/enterKing.h"
 #include <boost/scoped_ptr.hpp>
 #include <boost/thread/thread.hpp>
 #include <boost/thread/mutex.hpp>
@@ -66,6 +69,8 @@ int book_width_root = 16, book_width = 3;
 #else
 int book_width_root = 16, book_width = 10;
 #endif
+uint64_t node_count_hard_limit = 0;
+bool new_move_probability = true;
 
 bool is_verbose()
 {
@@ -98,7 +103,7 @@ struct UsiState
   {
     NumEffectState state(initial_state);
     BOOST_FOREACH(Move m, moves)
-      ApplyMoveOfTurn::doMove(state, m);
+      state.makeMove(m);
     return state;
   }
 };
@@ -155,7 +160,7 @@ public:
   {
     if (is_verbose())
       std::cerr << state.state() << "\n";
-    assert(state.state().getTurn() == my_turn);
+    assert(state.state().turn() == my_turn);
     if (initial_time[BLACK] < 0) initial_time[BLACK] = bwtime[BLACK];
     if (initial_time[WHITE] < 0) initial_time[WHITE] = bwtime[WHITE];
     MoveWithComment ret;
@@ -203,14 +208,17 @@ public:
       boost::mutex::scoped_lock lk(mutex);
       usi_state.aborted = true;
     }
+    player->setRootIgnoreMoves(0, false);
+    ignore_moves.clear();
+    OslConfig::resetRootWindow();
     std::string result = "bestmove " + record::usi::show(ret.move);
-    std::cout << result << "\n" << std::flush;
+    {
+      boost::mutex::scoped_lock lk(OslConfig::lock_io);
+      std::cout << result << "\n" << std::flush;
+    }
     if (is_verbose())
       std::cerr << "sent: " << result << "\n";
     suspend_reading = false;
-    player->setRootIgnoreMoves(0, false);
-    ignore_moves.clear();
-    OslConfig::resetRootNullWindow();
     boost::mutex::scoped_lock lk(go_stop_mutex);
     ++bestmove_count;
     go_stop_condition.notify_all();
@@ -300,6 +308,13 @@ boost::scoped_ptr<UsiState> input_state;
 boost::scoped_ptr<boost::thread> search_thread;
 
 game_playing::ComputerPlayer *makePlayer(Player turn, const SimpleState&);
+Player input_turn() 
+{
+  Player turn = input_state->initial_state.turn();
+  if (! input_state->moves.empty())
+    turn = alt(input_state->moves.back().player());
+  return turn;
+}
 void do_search()
 {
   if (search_thread && input_state->isSuccessorOf(player->current())) {
@@ -315,9 +330,7 @@ void do_search()
     while (queue.pop_if_present(dummy))
       ;
   }
-  Player turn = input_state->initial_state.getTurn();
-  if (! input_state->moves.empty())
-    turn = alt(input_state->moves.back().player());
+  const Player turn = input_turn();
   if (debug)
     std::cerr << "new game " << turn << "\n";
   if (csa_file != "") {
@@ -386,13 +399,17 @@ void GoMateThread::operator()()
   NotifyLock lock;
   NumEffectState state(input_state->currentState());
 #if (! defined ALLOW_KING_ABSENCE)
-  if (state.getKingPosition(state.getTurn()).isPieceStand()) {
+  if (state.kingSquare(state.turn()).isPieceStand()) {
+    boost::mutex::scoped_lock lk(OslConfig::lock_io);
     std::cout << "checkmate notimplemented\n";
     return;
   }
 #endif  
-  checkmate::DfpnTable table(state.getTurn());
-  const PathEncoding path(state.getTurn());
+  checkmate::DfpnTable table(state.turn());
+#ifndef MINIMAL
+  table.setGrowthLimit(3000000ull*OslConfig::memoryUseLimit()/(1024*1024*1024));
+#endif
+  const PathEncoding path(state.turn());
   Move checkmate_move;
   vector<Move> pv;
   ProofDisproof result;
@@ -407,20 +424,22 @@ void GoMateThread::operator()()
   MilliSeconds start = MilliSeconds::now();
 #ifdef OSL_DFPN_SMP
   boost::thread thread(GoMate(dfpn, state, path, checkmate_move, pv, result));
-  boost::this_thread::sleep(boost::posix_time::milliseconds(450));
+  double wait = 450;
   for (int i=0; true; ++i) {
+    if (thread.timed_join(boost::posix_time::milliseconds((size_t)wait)))
+      break;
     double elapsed = start.elapsedSeconds();
     size_t node_count = dfpn.nodeCount();
     if (i % 4 == 0) {
       double memory = OslConfig::memoryUseRatio();
+      boost::mutex::scoped_lock lk(OslConfig::lock_io);
       std::cout << "info time " << static_cast<int>(elapsed*1000)
 		<< " nodes " << node_count << " nps " << static_cast<int>(node_count/elapsed)
 		<< " hashfull " << static_cast<int>(memory*1000) << "\n";
     }
     if ((seconds*0.94-elapsed)*1000 < 110 || stop_checkmate)
       break;
-    double wait = std::min((seconds*0.94-elapsed)*1000 - 100, 250.0);
-    boost::this_thread::sleep(boost::posix_time::milliseconds((size_t)wait));
+    wait = std::min((seconds*0.94-elapsed)*1000 - 100, 250.0);
   }
   dfpn.stopNow();
   thread.join();
@@ -434,6 +453,7 @@ void GoMateThread::operator()()
     double memory = OslConfig::memoryUseRatio();
     uint64_t node_count = dfpn.nodeCount();
     total += limit;
+    boost::mutex::scoped_lock lk(OslConfig::lock_io);
     std::cout << "info time " << static_cast<int>(elapsed*1000)
 	      << " nodes " << node_count << " nps " << static_cast<int>(node_count/elapsed)
 	      << " hashfull " << static_cast<int>(memory*1000) << "\n";
@@ -448,17 +468,21 @@ void GoMateThread::operator()()
   if (is_verbose())
     std::cerr << "elapsed " << start.elapsedSeconds() << "\n";
   if (! result.isFinal()) {
-    std::cout << "checkmate timeout\n";
+    boost::mutex::scoped_lock lk(OslConfig::lock_io);
+    std::cout << "checkmate timeout\n" << std::flush;
     return;
   }
   if (! result.isCheckmateSuccess()) {
-    std::cout << "checkmate nomate\n";
+    boost::mutex::scoped_lock lk(OslConfig::lock_io);
+    std::cout << "checkmate nomate\n" << std::flush;
+;
     return;
   }
+  boost::mutex::scoped_lock lk(OslConfig::lock_io);
   std::cout << "checkmate";
   for (size_t i=0; i<pv.size(); ++i)
     std::cout << " " << record::usi::show(pv[i]);
-  std::cout << "\n";
+  std::cout << "\n" << std::flush;
 }
 
 void do_checkmate(double seconds)
@@ -484,6 +508,7 @@ void play_by_book(int depth_limit)
       const Move best_move = tracer.selectMove();
       if (best_move.isNormal()
 	  && (! state.isIllegal(best_move))) {
+	boost::mutex::scoped_lock lk(OslConfig::lock_io);
 	std::string result = "bestmove " + record::usi::show(best_move);
 	std::cout << result << "\n" << std::flush;
 	if (is_verbose())
@@ -492,11 +517,24 @@ void play_by_book(int depth_limit)
       }
     }
   }
+  boost::mutex::scoped_lock lk(OslConfig::lock_io);
   std::cout << "bestmove pass\n" << std::flush;
   if (is_verbose())
     std::cerr << "sent: " << "bestmove pass\n";
 }
 
+void play_declare_win()
+{
+  const bool can_declare_win
+    = EnterKing::canDeclareWin(input_state->currentState());
+  const std::string msg = std::string("bestmove ")
+    + (can_declare_win ? "win" : "pass") + "\n" ;
+  boost::mutex::scoped_lock lk(OslConfig::lock_io);
+  std::cout << msg << std::flush;
+  if (is_verbose())
+    std::cerr << "sent: " << msg;
+}
+
 MoveVector generate_good_moves()
 {
   game_playing::GameState state(input_state->initial_state);
@@ -505,9 +543,11 @@ MoveVector generate_good_moves()
 
   MoveVector normal_or_win_or_draw, loss;
   state.generateNotLosingMoves(normal_or_win_or_draw, loss);
-  if (is_verbose()) {
+  if (is_verbose() && ! loss.empty()) {
+    std::cerr << "  removed losing move ";
     BOOST_FOREACH(Move m, loss)
-      std::cerr << "  removed losing move " << record::csa::show(m) << "\n";
+      std::cerr << record::csa::show(m);
+    std::cerr << "\n";
   }
   return normal_or_win_or_draw;
 }
@@ -515,6 +555,7 @@ MoveVector generate_good_moves()
 void genmove()
 {
   MoveVector moves = generate_good_moves();
+  boost::mutex::scoped_lock lk(OslConfig::lock_io);
   std::cout << "genmove";
   BOOST_FOREACH(Move move, moves)
     std::cout << " " << record::usi::show(move);
@@ -523,26 +564,46 @@ void genmove()
 
 void genmove_probability(int limit)
 {
-  MoveVector good_moves = generate_good_moves();
-  NumEffectState state(input_state->currentState());
-  const rating::StandardFeatureSet& feature_set
-    = rating::StandardFeatureSet::instance();
-  std::cout << "genmove_probability";
-  MoveLogProbVector moves;
-  rating::RatingEnv env;
+  game_playing::GameState gstate(input_state->initial_state);
+  BOOST_FOREACH(Move m, input_state->moves) 
+    gstate.pushMove(m);
+  const NumEffectState& state= gstate.state();
+  const MoveStack& history = gstate.moveHistory();
   progress::ml::NewProgress progress(state);
-  env.make(state, state.pin(state.getTurn()), state.pin(alt(state.getTurn())),
-	   progress.progress16());
-  feature_set.generateLogProbNormalized(state, env, limit, moves);
+  MoveLogProbVector moves;
+  if (new_move_probability) {
+    const move_probability::StandardFeatureSet& feature_set
+      = move_probability::StandardFeatureSet::instance();
+    Move threatmate = move_probability::StateInfo::findShortThreatmate
+      (state, history.lastMove());
+    move_probability::StateInfo info(state, progress.progress16(),
+				     history, threatmate);
+    feature_set.generateLogProb(info, moves);
+  } else {
+    const rating::StandardFeatureSet& feature_set
+      = rating::StandardFeatureSet::instance();
+    rating::RatingEnv env;
+    env.history = history;
+    env.make(state, state.pin(state.turn()), state.pin(alt(state.turn())),
+	     progress.progress16());
+    feature_set.generateLogProb(state, env, limit, moves);
+  }
+  for (size_t i=1; i<moves.size(); ++i)
+    if (moves[i].logProb() <= moves[i-1].logProb() && moves[i-1].logProb()+1<=limit)
+      moves[i].setLogProb(moves[i-1].logProb()+1);
+  MoveVector good_moves = generate_good_moves();
+  boost::mutex::scoped_lock lk(OslConfig::lock_io);
+  std::cout << "genmove_probability";
   BOOST_FOREACH(MoveLogProb move, moves)
-    if (good_moves.isMember(move.getMove()))
-      std::cout << " " << record::usi::show(move.getMove()) << " " << move.getLogProb();
+    if (good_moves.isMember(move.move()))
+      std::cout << " " << record::usi::show(move.move()) << " " << move.logProb();
   std::cout << "\n";
 }
 
 void csashow()
 {
   NumEffectState state(input_state->currentState());
+  boost::mutex::scoped_lock lk(OslConfig::lock_io);
   std::cout << state;
   std::cout << "csashowok\n";
 }
@@ -551,6 +612,7 @@ void csamove(std::string str)
 {
   const NumEffectState state(input_state->currentState());
   const Move move = record::usi::strToMove(str, state);
+  boost::mutex::scoped_lock lk(OslConfig::lock_io);
   std::cout << "csamove " << record::csa::show(move) << "\n";
 }
 
@@ -845,8 +907,10 @@ void play(std::istream& is, std::ostream& os)
       }
       if (last_go_is_checkmate)
 	stop_checkmate = true;
-      else
+      else {
 	player->stopSearchNow();
+	player->needRestartNext();
+      }
       boost::mutex::scoped_lock lk(go_stop_mutex);
       while (go_count > bestmove_count)
 	go_stop_condition.wait(lk);
@@ -859,6 +923,8 @@ void play(std::istream& is, std::ostream& os)
       }
       return;
     }
+
+    // extended commands
     if (line.find("sleep") == 0) {
       boost::this_thread::sleep(boost::posix_time::seconds(60));
       continue;
@@ -883,7 +949,14 @@ void play(std::istream& is, std::ostream& os)
       csamove(line.substr(8));
       continue;
     }
+    if (line.find("echo ") == 0) {
+      boost::mutex::scoped_lock lk(OslConfig::lock_io);
+      std::cout << line.substr(5) << "\n";
+      std::cerr << line.substr(5) << "\n";
+      continue;
+    }
 
+    // searc related commands 
     // do not read next search command while previous search is in progress
     while (read_from_file && suspend_reading)
       boost::this_thread::sleep(boost::posix_time::milliseconds(200));
@@ -905,6 +978,7 @@ void play(std::istream& is, std::ostream& os)
     }
     if (line.find("go") == 0) {
       boost::mutex::scoped_lock lk(go_stop_mutex);
+      OslConfig::setDfpnMaxDepth(256);
       while (go_count > bestmove_count) {
 	std::cerr << "delay go (go " << go_count
 		  << " times, bestmove " << bestmove_count << " times)\n";
@@ -922,6 +996,7 @@ void play(std::istream& is, std::ostream& os)
 	boost::mutex::scoped_lock lk(go_stop_mutex);
 	++go_count;
 	last_go_is_checkmate = true;
+	OslConfig::setDfpnMaxDepth(1600);
       }
       do_checkmate(seconds);
       continue;
@@ -934,38 +1009,48 @@ void play(std::istream& is, std::ostream& os)
       play_by_book(limit);
       continue;
     }
-    if (line.find("go nullwindow ") == 0) {
+    if (line.find("go declare_win") == 0) {
+      play_declare_win();
+      continue;
+    }
+    if (line.find("go window ") == 0) {
       if (ponder_enabled) {
 	std::string err
-	  = "go nullwindow not supported with ponder enabled";
+	  = "go window not supported if ponder enabled";
 	std::cerr << err << "\n";
 	  throw std::runtime_error(err);
       }
       bwtime.fill(1000);
       byoyomi = 1000;
       std::string option;
-      std::istringstream is(line.substr(strlen("go nullwindow ")));
-      int null_window;
-      if (! (is >> null_window)) {
+      std::istringstream is(line.substr(strlen("go window ")));
+      int alpha, beta;
+      if (! (is >> alpha >> beta)) {
 	std::string err
-	  = "go nullwindow: window error ";
+	  = "go window: window error ";
 	std::cerr << err << "\n";
 	  throw std::runtime_error(err);
       }
-      std::cerr <<"hello nullwindow " << null_window << " " << (bool)is << "\n";
-      
+      std::cerr <<"window is " << alpha << ' ' << beta << " "
+		<< (bool)is << ' ' << input_turn() << "\n";
+      if (input_turn() == WHITE) {
+	std::swap(alpha, beta);
+	alpha = -alpha;
+	beta = -beta;
+	std::cerr <<"window adjustment " << alpha << ' ' << beta << " " << (bool)is << "\n";
+      }
       while (is >> option) {
 	if (option == "byoyomi")
 	  is >> byoyomi;
 	else {
 	  std::string err
-	    = "unknown option in go null_window " + option;
+	    = "unknown option in go window " + option;
 	  std::cerr << err << "\n";
 	  throw std::runtime_error(err);
 	}
       }
       byoyomi_extension = byoyomi;
-      OslConfig::setRootNullWindow(null_window);
+      OslConfig::setRootWindow(alpha, beta);
       suspend_reading = true;
       do_search();
       boost::mutex::scoped_lock lk(go_stop_mutex);
@@ -1028,6 +1113,8 @@ void play(std::istream& is, std::ostream& os)
  	ignore_moves.push_back(record::usi::strToMove(word, state));
       }
       std::cerr << "accept ignore_moves\n";
+      if (player)
+	player->needRestartNext();
       continue;
     }
     if (line.find("query progress") == 0) {
@@ -1035,12 +1122,14 @@ void play(std::istream& is, std::ostream& os)
       if (input_state) {
 	NumEffectState state(input_state->currentState());
 	const osl::progress::Effect5x3 progress(state);
+	boost::mutex::scoped_lock lk(OslConfig::lock_io);
 	os << "answer progress " << progress.progress16().value()
 	   << ' ' << progress.progress16(BLACK).value()
 	   << ' ' << progress.progress16(WHITE).value()
 	   << "\n";
       }
       else {
+	boost::mutex::scoped_lock lk(OslConfig::lock_io);
 	os << "error position not specified\n";
       }
       continue;
@@ -1050,6 +1139,7 @@ void play(std::istream& is, std::ostream& os)
       if (input_state) {
 	NumEffectState state(input_state->currentState());
 	const osl::eval::ml::OpenMidEndingEval eval(state);
+	boost::mutex::scoped_lock lk(OslConfig::lock_io);
 	os << "answer eval " << eval.value()
 	   << ' ' << eval.captureValue(osl::newPtypeO(osl::WHITE,osl::PAWN))/2
 	   << ' ' << eval.progressIndependentValue()
@@ -1060,6 +1150,7 @@ void play(std::istream& is, std::ostream& os)
 	   << "\n";
       }
       else {
+	boost::mutex::scoped_lock lk(OslConfig::lock_io);
 	os << "error position not specified\n";
       }
       continue;
@@ -1076,10 +1167,12 @@ void play(std::istream& is, std::ostream& os)
 	    state.pushMove(m);
 	  const osl::search::TimeAssigned assigned
 	    = osl::game_playing::SearchPlayer::assignTime(state, amount, elapsed, byoyomi, 0);
+	  boost::mutex::scoped_lock lk(OslConfig::lock_io);
 	  os << "answer searchtime " << assigned.standard.value()
 	     << ' ' << assigned.max.value() << "\n";
 	}
 	else {
+	  boost::mutex::scoped_lock lk(OslConfig::lock_io);
 	  os << "error in arguments";
 	}
       }
@@ -1089,9 +1182,43 @@ void play(std::istream& is, std::ostream& os)
       continue;      
     }
     if (line.find("query") == 0) {
+      boost::mutex::scoped_lock lk(OslConfig::lock_io);
       os << "error unknown query\n";
       continue;      
     }    
+    if (line.find("open ") == 0) {
+      if (! input_state)
+	input_state.reset(new UsiState);
+      std::istringstream is(line);
+      std::string dummy, filename;
+      bool ok = true;
+      ok &= static_cast<bool>(is >> dummy >> filename);
+      if (ok) {
+	try {
+	  CsaFile csa(filename.c_str());
+	  input_state->initial_state = csa.getInitialState();
+	  input_state->moves = csa.getRecord().getMoves();
+	}
+	catch (record::csa::CsaIOError& e) {
+	  std::cerr << e.what() << "\n";
+	  ok = false;
+	}
+      }
+      if (ok) {
+	boost::mutex::scoped_lock lk(OslConfig::lock_io);
+	std::cout << "position "
+		  << record::usi::show(input_state->initial_state)
+		  << " moves";
+	BOOST_FOREACH(Move move, input_state->moves)
+	  std::cout << " " << record::usi::show(move);
+	std::cout << "\n";
+	std::cout << "openok\n";	
+      }
+      if (! ok)
+	std::cerr << "open failed " << line << "\n";
+      continue;
+    }
+    std::cerr << "unknown command " << line << "\n";
   }
   if (read_from_file) {
     std::cerr << "end of input\n";
@@ -1108,12 +1235,15 @@ void play(std::istream& is, std::ostream& os)
 int main(int argc, char **argv)
 {
   int num_benchmark, benchmark_seconds;
-  bool use_log_linear_probability;
   po::options_description options("Options");
   std::string input_file;
+#ifndef MINIMAL
+  unsigned int eval_random;
+#endif
 #ifdef OSL_SMP
   int num_cpus;
 #endif
+  double memory_use_percent;
   options.add_options()
     ("benchmark", "test search performance")
     ("benchmark-single", "test search performance")
@@ -1131,14 +1261,26 @@ int main(int argc, char **argv)
     ("book-width-root", 
      po::value<int>(&book_width_root)->default_value(book_width_root),
      "relative width of book at the initial position")
+#ifndef MINIMAL
+    ("eval-randomness",
+     po::value<unsigned int>(&eval_random)->default_value(0),
+     "add random value generated by normal distribution of a given standard deviation, to evaluation values")
+    ("node-count-hard-limit",
+     po::value<uint64_t>(&node_count_hard_limit)->default_value(0),
+     "nodes limit for each search (0 for infinity)")
+#endif
 #ifdef OSL_SMP
     ("num-cpus,N",
      po::value<int>(&num_cpus)->default_value(-1),
      "num cpus for parallel search")
 #endif
-    ("use-log-linear-probability",
-     po::value<bool>(&use_log_linear_probability)->default_value(false),
-     "alternative configuration for realization probability search (experimental)")
+    ("new-move-probability-in-genmove-logprobability",
+     po::value<bool>(&new_move_probability)->default_value(true),
+     "use new (experimental) move probability for genmove_probability")
+    ("memory-use-percent",
+     po::value<double>(&memory_use_percent)->default_value(100.0),
+     "percentage for memory use (normally 100)")
+    ("health-check", "test whether data files are properly placed")
     ("help,h", "produce help message")
     ("version", "show version info")
     ;
@@ -1155,23 +1297,43 @@ int main(int argc, char **argv)
     std::cerr << options << std::endl;
     throw;
   }
+  if (vm.count("health-check")) {
+    bool ok = OslConfig::healthCheck();
+    ok &= rating::StandardFeatureSet::healthCheck();
+    ok &= move_probability::StandardFeatureSet::healthCheck();
+    if (! ok) 
+      std::cerr << "health check failed\n";
+    return ok ? 0 : 1;
+  }
   if (vm.count("help")) {
     std::cout << options << std::endl;
     return 0;
   }
   if (vm.count("version")) {
     std::cout << "gpsusi " << gpsshogi::gpsshogi_revision << "\n\n"
-      << "Copyright (C) 2003-2009 Team GPS, the University of Tokyo.\n";
+      << "Copyright (C) 2003-2011 Team GPS.\n";
     return 0;
   }
+  osl::OslConfig::setMemoryUsePercent(memory_use_percent);
+#ifndef MINIMAL
+  if (eval_random)
+    OslConfig::setEvalRandom(eval_random);
+#endif
   if (vm.count("benchmark")
       || vm.count("benchmark-single")
       || num_benchmark) {
+#ifdef OSL_SMP
+    if (num_cpus > 0 && num_cpus != OslConfig::numCPUs()
+	&& ! vm.count("profile")) {
+      std::cerr << "set num-cpus " << OslConfig::numCPUs() << " => " << num_cpus << "\n";
+      OslConfig::setNumCPUs(num_cpus);
+    }
+#endif
     eval::ml::OpenMidEndingEval::setUp();
     progress::ml::NewProgress::setUp();
     rating::StandardFeatureSet::instance();
   }
-  if (vm.count("benchmark")) {
+  if (vm.count("benchmark") || vm.count("benchmark-single")) {
     game_playing::AlphaBeta2OpenMidEndingEvalPlayer player;
     player.setDepthLimit(2000, 400, 200);
     player.setNodeLimit(std::numeric_limits<size_t>::max());
@@ -1179,7 +1341,6 @@ int main(int argc, char **argv)
     benchmark(player, "", benchmark_seconds);
     return 0;
   }
-  OslConfig::enableLogLinearProbability(use_log_linear_probability);
   if (num_benchmark) {
 #ifdef OSL_SMP
     if (vm.count("profile"))
@@ -1194,16 +1355,13 @@ int main(int argc, char **argv)
   }
 
   OslConfig::setUsiMode();
+  OslConfig::setSearchExactValueInOneReply(true); // todo: option
   read_config_file();
 #ifdef OSL_SMP
   if (num_cpus > 0 && num_cpus != OslConfig::numCPUs()) {
     std::cerr << "set num-cpus " << OslConfig::numCPUs() << " => " << num_cpus << "\n";
     OslConfig::setNumCPUs(num_cpus);
   }
-#endif
-#if OSL_WORDSIZE == 32
-  if (! OslConfig::isMemoryLimitEffective())
-    osl::OslConfig::setNumCPUs(std::min(2, osl::OslConfig::numCPUs()));
 #endif
   setvbuf(stdout, NULL, _IONBF, 0);
   setvbuf(stdin, NULL, _IONBF, 0);
@@ -1249,6 +1407,10 @@ game_playing::ComputerPlayer *makePlayer(Player turn, const SimpleState& initial
   }
   search_player->setVerbose(is_verbose() ? 2 : 0);
   search_player->setDepthLimit(limit_depth*200, 400, 200);
+#ifndef MINIMAL
+  if (node_count_hard_limit)
+    search_player->setNodeCountHardLimit(node_count_hard_limit);
+#endif
   bool use_book = book_depth > 0;
   {
     const SimpleState usual(HIRATE);
