@@ -12,9 +12,9 @@ getopts("l:h:r:",\%options);
 my ($localdir, $host, $remotedir, $lpath, $rpath)
   = ($options{l} || "local-csa",
      $options{h} || "macpro",
-     $options{r} || "remote-csa",
+     $options{r} || "/home/wcsc21/remote-csa",
      $options{L} || "..",
-     $options{R} || "/home/kaneko/up/gpsshogi",
+     $options{R} || "/home/wcsc21",
     );
 
 system "mkdir", "-p", "$localdir/in";
