@@ -19,14 +19,14 @@ namespace gpsshogi
       // throws an exception
       osl::state::SimpleState getState();
       void setState(const osl::state::SimpleState &state);
-      void moveReserve(osl::Player, osl::Ptype, osl::Position);
+      void moveReserve(osl::Player, osl::Ptype, osl::Square);
       void reserveToStand(osl::Player);
     public slots:
       void clearState();
     protected:
-      virtual osl::Piece getPieceOf(int i);
+      virtual osl::Piece pieceOf(int i);
       virtual int countPiecesOnStand(osl::Player, osl::Ptype);
-      virtual osl::Piece getPieceAt(osl::Position);
+      virtual osl::Piece pieceAt(osl::Square);
       virtual osl::Piece getReservePiece(osl::Ptype);
       virtual osl::Piece getStandPiece(osl::Player, osl::Ptype);
       void mousePressEvent(QMouseEvent *);
@@ -35,7 +35,7 @@ namespace gpsshogi
       void keyReleaseEvent(QKeyEvent *);
     private:
       template<osl::Ptype ptype> void initPieces();
-      osl::misc::CArray<osl::Piece, osl::Position::SIZE> board;
+      osl::misc::CArray<osl::Piece, osl::Square::SIZE> board;
       osl::misc::CArray<osl::Piece, osl::Piece::SIZE> pieces;
       int modifiers;
     };
