@@ -39,7 +39,7 @@ namespace gpsshogi
       void remove_piece(osl::Piece);
 
       void place_piece(osl::Player,
-		       osl::Position,
+		       osl::Square,
 		       osl::Ptype);
 
       void update_state(osl::Piece pieces[40]);
