@@ -23,8 +23,8 @@ namespace gpsshogi
   public:
     enum { ONE_DIM = 2142 };
     PieceKingRelativeFeature() : FeaturesOneNonUniq(ONE_DIM * 2) { }
-    static int index(const Player player, const Position king,
-		     const Ptype ptype, const Position pos)
+    static int index(const Player player, const Square king,
+		     const Ptype ptype, const Square pos)
     {
       const int x = std::abs(pos.x() - king.x());
       const int y = (king.y() - pos.y()) *
@@ -32,10 +32,10 @@ namespace gpsshogi
       return (ptype - osl::PTYPE_PIECE_MIN) * 17 * 9 + (x * 17 + y);
     }
 
-    static int index(const Player player, const Position king,
+    static int index(const Player player, const Square king,
 		     const Piece piece)
     {
-      return index(player, king, piece.ptype(), piece.position());
+      return index(player, king, piece.ptype(), piece.square());
     }
     size_t maxActive() const { return 160; }
     const std::string name() const { return "PieceKingRelativeBoth"; };
@@ -68,13 +68,13 @@ namespace gpsshogi
   {
   protected:
     virtual int distanceIndex(const osl::Player player,
-			      const osl::Position king,
-			      const osl::Position pos) const = 0;
+			      const osl::Square king,
+			      const osl::Square pos) const = 0;
     virtual int index(const osl::Ptype ptype,
 		      int distanceIndex) const = 0;
     int index(const osl::Player player,
-	      const osl::Position king,
-	      const osl::Position pos,
+	      const osl::Square king,
+	      const osl::Square pos,
 	      const osl::Ptype ptype) const {
       return  index(ptype, distanceIndex(player, king, pos));
     }
@@ -90,7 +90,7 @@ namespace gpsshogi
 		  int offset) const;
     int pieceValue(const osl::state::NumEffectState &state,
 		   const osl::Piece piece) const;
-    int pieceValue(const osl::Position king,const osl::Piece piece) const;
+    int pieceValue(const osl::Square king,const osl::Piece piece) const;
     size_t maxActive() const { return 38; }
     void showSummary(std::ostream &os) const;
     void showAll(std::ostream& os) const;
@@ -100,8 +100,8 @@ namespace gpsshogi
   {
   protected:
     int distanceIndex(const osl::Player player,
-		      const osl::Position king,
-		      const osl::Position pos) const;
+		      const osl::Square king,
+		      const osl::Square pos) const;
     int index(const osl::Ptype ptype,
 	      int distanceIndex) const;
   public:
@@ -121,10 +121,10 @@ namespace gpsshogi
   protected:
     int index(const NumEffectState &state, const Piece piece) const
     {
-      return index(state, piece.ptypeO(), piece.position());
+      return index(state, piece.ptypeO(), piece.square());
     }
     int index(const NumEffectState &state,
-		      const PtypeO ptypeO, const Position position) const;
+		      const PtypeO ptypeO, const Square position) const;
     bool is_attack;
   public:
     PieceKingRelativeAbsBase(bool attack=true)
@@ -174,10 +174,10 @@ namespace gpsshogi
   protected:
     int index(const NumEffectState &state, const Piece piece) const
     {
-      return index(state, piece.ptypeO(), piece.position());
+      return index(state, piece.ptypeO(), piece.square());
     }
     int index(const NumEffectState &state,
-	      const PtypeO ptypeO, const Position position) const;
+	      const PtypeO ptypeO, const Square position) const;
     bool is_attack;
   public:
     PieceKingRelativeAbs(bool attack=true)
@@ -196,10 +196,10 @@ namespace gpsshogi
 
   class SimpleAttackKing : public EvalComponent
   {
-    int distanceIndex(const osl::Position king,
-		      const osl::Position pos) const;
-    int index(const osl::Position king,
-	      const osl::Position pos,
+    int distanceIndex(const osl::Square king,
+		      const osl::Square pos) const;
+    int index(const osl::Square king,
+	      const osl::Square pos,
 	      const osl::Ptype ptype) const;
     int index(const osl::Ptype ptype,
 	      int distanceIndex) const;
@@ -250,11 +250,11 @@ namespace gpsshogi
     virtual const std::string name() const { return "King8EffectBase"; };
   };
 
-  class King8EffectEmptyPosition : public King8EffectBase
+  class King8EffectEmptySquare : public King8EffectBase
   {
   public:
-    King8EffectEmptyPosition() : King8EffectBase(8 * 4) { }
-    const std::string name() const { return "King8EffectEmptyPosition"; };
+    King8EffectEmptySquare() : King8EffectBase(8 * 4) { }
+    const std::string name() const { return "King8EffectEmptySquare"; };
     virtual void showSummary(std::ostream &os) const;
   protected:
     virtual int index(Piece king,
@@ -262,11 +262,11 @@ namespace gpsshogi
 		      EffectState state) const;
   };
 
-  class King8EffectEmptyPositionY : public King8EffectBase
+  class King8EffectEmptySquareY : public King8EffectBase
   {
   public:
-    King8EffectEmptyPositionY() : King8EffectBase(8 * 4 * 9) { }
-    const std::string name() const { return "King8EffectEmptyPositionY"; };
+    King8EffectEmptySquareY() : King8EffectBase(8 * 4 * 9) { }
+    const std::string name() const { return "King8EffectEmptySquareY"; };
     void showAll(std::ostream &os) const;
   protected:
     virtual int index(Piece king,
@@ -274,23 +274,23 @@ namespace gpsshogi
 		      EffectState state) const;
   };
 
-  class King8EffectDefensePositionY : public King8EffectEmptyPositionY
+  class King8EffectDefenseSquareY : public King8EffectEmptySquareY
   {
   public:
-    King8EffectDefensePositionY() : King8EffectEmptyPositionY() { }
+    King8EffectDefenseSquareY() : King8EffectEmptySquareY() { }
   protected:
     virtual EffectState effectState(const NumEffectState &state,
 				    const Player defense,
 				    const Direction dir) const;
-    const std::string name() const { return "King8EffectDefensePositionY"; };
+    const std::string name() const { return "King8EffectDefenseSquareY"; };
   };
 
-  class King8EffectEmptyPositionDBlocked : public King8EffectEmptyPosition
+  class King8EffectEmptySquareDBlocked : public King8EffectEmptySquare
   {
   public:
-    King8EffectEmptyPositionDBlocked() : King8EffectEmptyPosition() { }
+    King8EffectEmptySquareDBlocked() : King8EffectEmptySquare() { }
     const std::string name() const {
-      return "King8EffectEmptyPositionDBlocked";
+      return "King8EffectEmptySquareDBlocked";
     };
     static bool isBlocked(const osl::checkmate::King8Info &);
   protected:
@@ -298,12 +298,12 @@ namespace gpsshogi
       return isBlocked(info);
     }
   };
-  class King8EffectEmptyPositionUBlocked : public King8EffectEmptyPosition
+  class King8EffectEmptySquareUBlocked : public King8EffectEmptySquare
   {
   public:
-    King8EffectEmptyPositionUBlocked() : King8EffectEmptyPosition() { }
+    King8EffectEmptySquareUBlocked() : King8EffectEmptySquare() { }
     const std::string name() const {
-      return "King8EffectEmptyPositionUBlocked";
+      return "King8EffectEmptySquareUBlocked";
     };
     static bool isBlocked(const osl::checkmate::King8Info &);
   protected:
@@ -311,12 +311,12 @@ namespace gpsshogi
       return isBlocked(info);
     }
   };
-  class King8EffectEmptyPositionLBlocked : public King8EffectEmptyPosition
+  class King8EffectEmptySquareLBlocked : public King8EffectEmptySquare
   {
   public:
-    King8EffectEmptyPositionLBlocked() : King8EffectEmptyPosition() { }
+    King8EffectEmptySquareLBlocked() : King8EffectEmptySquare() { }
     const std::string name() const {
-      return "King8EffectEmptyPositionLBlocked";
+      return "King8EffectEmptySquareLBlocked";
     };
     static bool isBlocked(const osl::checkmate::King8Info &);
   protected:
@@ -324,12 +324,12 @@ namespace gpsshogi
       return isBlocked(info);
     }
   };
-  class King8EffectEmptyPositionRBlocked : public King8EffectEmptyPosition
+  class King8EffectEmptySquareRBlocked : public King8EffectEmptySquare
   {
   public:
-    King8EffectEmptyPositionRBlocked() : King8EffectEmptyPosition() { }
+    King8EffectEmptySquareRBlocked() : King8EffectEmptySquare() { }
     const std::string name() const {
-      return "King8EffectEmptyPositionRBlocked";
+      return "King8EffectEmptySquareRBlocked";
     };
     static bool isBlocked(const osl::checkmate::King8Info &);
   protected:
@@ -338,63 +338,63 @@ namespace gpsshogi
     }
   };
 
-  class King8EffectDefensePosition : public King8EffectEmptyPosition
+  class King8EffectDefenseSquare : public King8EffectEmptySquare
   {
   public:
-    King8EffectDefensePosition() : King8EffectEmptyPosition() { }
+    King8EffectDefenseSquare() : King8EffectEmptySquare() { }
   protected:
     virtual EffectState effectState(const NumEffectState &state,
 			    const Player defense,
 			    const Direction dir) const;
-    const std::string name() const { return "King8EffectDefensePosition"; };
+    const std::string name() const { return "King8EffectDefenseSquare"; };
   };
 
-  class King8EffectDefensePositionDBlocked : public King8EffectDefensePosition
+  class King8EffectDefenseSquareDBlocked : public King8EffectDefenseSquare
   {
   public:
-    King8EffectDefensePositionDBlocked() : King8EffectDefensePosition() { }
+    King8EffectDefenseSquareDBlocked() : King8EffectDefenseSquare() { }
     const std::string name() const {
-      return "King8EffectDefensePositionDBlocked";
+      return "King8EffectDefenseSquareDBlocked";
     };
   protected:
     virtual bool isTarget(const osl::checkmate::King8Info &info) const {
-      return King8EffectEmptyPositionDBlocked::isBlocked(info);
+      return King8EffectEmptySquareDBlocked::isBlocked(info);
     }
   };
-  class King8EffectDefensePositionUBlocked : public King8EffectDefensePosition
+  class King8EffectDefenseSquareUBlocked : public King8EffectDefenseSquare
   {
   public:
-    King8EffectDefensePositionUBlocked() : King8EffectDefensePosition() { }
+    King8EffectDefenseSquareUBlocked() : King8EffectDefenseSquare() { }
     const std::string name() const {
-      return "King8EffectDefensePositionUBlocked";
+      return "King8EffectDefenseSquareUBlocked";
     };
   protected:
     virtual bool isTarget(const osl::checkmate::King8Info &info) const {
-      return King8EffectEmptyPositionUBlocked::isBlocked(info);
+      return King8EffectEmptySquareUBlocked::isBlocked(info);
     }
   };
-  class King8EffectDefensePositionLBlocked : public King8EffectDefensePosition
+  class King8EffectDefenseSquareLBlocked : public King8EffectDefenseSquare
   {
   public:
-    King8EffectDefensePositionLBlocked() : King8EffectDefensePosition() { }
+    King8EffectDefenseSquareLBlocked() : King8EffectDefenseSquare() { }
     const std::string name() const {
-      return "King8EffectDefensePositionLBlocked";
+      return "King8EffectDefenseSquareLBlocked";
     };
   protected:
     virtual bool isTarget(const osl::checkmate::King8Info &info) const {
-      return King8EffectEmptyPositionLBlocked::isBlocked(info);
+      return King8EffectEmptySquareLBlocked::isBlocked(info);
     }
   };
-  class King8EffectDefensePositionRBlocked : public King8EffectDefensePosition
+  class King8EffectDefenseSquareRBlocked : public King8EffectDefenseSquare
   {
   public:
-    King8EffectDefensePositionRBlocked() : King8EffectDefensePosition() { }
+    King8EffectDefenseSquareRBlocked() : King8EffectDefenseSquare() { }
     const std::string name() const {
-      return "King8EffectDefensePositionRBlocked";
+      return "King8EffectDefenseSquareRBlocked";
     };
   protected:
     virtual bool isTarget(const osl::checkmate::King8Info &info) const {
-      return King8EffectEmptyPositionRBlocked::isBlocked(info);
+      return King8EffectEmptySquareRBlocked::isBlocked(info);
     }
   };
 
@@ -532,7 +532,7 @@ namespace gpsshogi
       int &king_y)
     {
       const Player king_player = alt(attack);
-      const Position king = state.getKingPosition(king_player);
+      const Square king = state.kingSquare(king_player);
       if (king_player == BLACK)
 	king_y = king.y();
       else
@@ -548,13 +548,13 @@ namespace gpsshogi
       {
 	for (int x = min_x; x <= max_x; ++x)
 	{
-	  const Position target(x, y);
+	  const Square target(x, y);
 	  count += state.countEffect(attack, target);
-	  mask = mask | state.getEffect(target);
+	  mask = mask | state.effectSetAt(target);
 	}
       }
       effect = std::min(63, count);
-      mask = mask & state.getOnBoardMask(attack);
+      mask = mask & state.piecesOnBoard(attack);
       piece = std::min(16,  mask.countBit());
     }
   };
@@ -593,7 +593,7 @@ namespace gpsshogi
       int &king_y)
     {
       const Player king_player = alt(attack);
-      const Position king = state.getKingPosition(king_player);
+      const Square king = state.kingSquare(king_player);
       if (king_player == BLACK)
 	king_y = king.y();
       else
@@ -608,11 +608,11 @@ namespace gpsshogi
       {
 	for (int x = min_x; x <= max_x; ++x)
 	{
-	  const Position target(x, y);
-	  mask = mask | state.getEffect(target);
+	  const Square target(x, y);
+	  mask = mask | state.effectSetAt(target);
 	}
       }
-      mask = mask & state.getOnBoardMask(attack);
+      mask = mask & state.piecesOnBoard(attack);
       piece = std::min(16,  mask.countBit());
       mask = mask & state.effectedMask(attack);
       supported = std::min(16, mask.countBit());
@@ -642,7 +642,7 @@ namespace gpsshogi
     void stateOne(const NumEffectState &state,
 		  CArray<int, DIM> &result) const;
     virtual bool isTarget(const NumEffectState &state,
-			  const Position pos,
+			  const Square pos,
 			  const Player defense) const;
   public:
     King25Empty() : EvalComponent(DIM) { }
@@ -658,7 +658,7 @@ namespace gpsshogi
   {
   protected:
     virtual bool isTarget(const NumEffectState &state,
-			  const Position pos,
+			  const Square pos,
 			  const Player defense) const;
   public:
     King25EmptyNoEffect() : King25Empty() { }
@@ -669,8 +669,8 @@ namespace gpsshogi
   {
   private:
     enum { DIM = 5 * 5 * 5 * 9 };
-    int index(Position king,
-	      Position target,
+    int index(Square king,
+	      Square target,
 	      Player player) const;
     template <Player Defense>
     int evalOne(const osl::state::NumEffectState &state) const;
@@ -707,7 +707,7 @@ namespace gpsshogi
   private:
     // rel_y, abs_x, effect state, owner of piece
     enum { DIM = 5 * 3 * STATE_MAX * 3 };
-    static int index(const NumEffectState &state, Position target,
+    static int index(const NumEffectState &state, Square target,
 	      Player defense);
     template <osl::Player Defense, typename Integer, class Weights>
     static Integer evalOne(const osl::state::NumEffectState &state, Integer initial, const Weights&);
@@ -724,7 +724,7 @@ namespace gpsshogi
 		  index_list_t &diffs,
 		  int offset) const;
     static EffectState effectState(const NumEffectState &state,
-				   Position target,
+				   Square target,
 				   Player defense);
     const std::string name() const { return "King25EffectEach"; }
   };
@@ -734,11 +734,11 @@ namespace gpsshogi
   public:
     // rel_y, abs_x, effect state, owner of piece, king_y
     enum { DIM = 5 * 3 * King25EffectEach::STATE_MAX * 3 * 9 }; // 3240
-    static int index(const NumEffectState &state, Position target,
+    static int index(const NumEffectState &state, Square target,
 		     Player defense)
     {
-      const Position king = state.getKingPosition(defense);
-      const Piece piece = state.getPieceAt(target);
+      const Square king = state.kingSquare(defense);
+      const Piece piece = state.pieceAt(target);
       // [0, 2]
       const int rel_x = std::abs(king.x() - target.x());
       // [-2, +2]
@@ -797,11 +797,11 @@ namespace gpsshogi
   public:
     // rel_y, rel_x, effect state, owner of piece, king_x
     enum { DIM = 5 * 5 * King25EffectEach::STATE_MAX * 3 * 5 }; // 3000
-    static int index(const NumEffectState &state, Position target,
+    static int index(const NumEffectState &state, Square target,
 		     Player defense)
     {
-      const Position king = state.getKingPosition(defense);
-      const Piece piece = state.getPieceAt(target);
+      const Square king = state.kingSquare(defense);
+      const Piece piece = state.pieceAt(target);
       // [-2, 2]
       int rel_x = king.x() - target.x();
       // [-2, +2]
@@ -859,11 +859,11 @@ namespace gpsshogi
   class King25EffectEachXYFeatures : public FeaturesOneNonUniq
   {
   private:
-    static int index(const NumEffectState &state, Position target,
+    static int index(const NumEffectState &state, Square target,
 		     Player defense)
     {
-      const Position king = state.getKingPosition(defense);
-      const Piece piece = state.getPieceAt(target);
+      const Square king = state.kingSquare(defense);
+      const Piece piece = state.pieceAt(target);
       // [-2, 2]
       int rel_x = king.x() - target.x();
       // [-2, +2]
@@ -970,7 +970,7 @@ namespace gpsshogi
     const std::string name() const { return "KingXBothBlocked3"; }
   private:
     template <Player P>
-    int index(const Position king, bool is_l,
+    int index(const Square king, bool is_l,
 	      bool u_blocked, bool opp_u_blocked, bool opp_blocked) const
     {
       int x = king.x();
@@ -1000,7 +1000,7 @@ namespace gpsshogi
     const std::string name() const { return "KingXBothBlocked3Y"; }
   private:
     template <Player P>
-    int index(const Position king, bool is_l,
+    int index(const Square king, bool is_l,
 	      bool u_blocked, bool opp_u_blocked, bool opp_blocked) const
     {
       int x = king.x();
@@ -1025,8 +1025,8 @@ namespace gpsshogi
   {
   private:
     enum { DIM = 4 };
-    static int index(const NumEffectState &state, Position king,
-		     Position target);
+    static int index(const NumEffectState &state, Square king,
+		     Square target);
     template <Player Defense>
     void featureOne(const NumEffectState &state,
 		    CArray<int, DIM> &features) const;
@@ -1045,18 +1045,18 @@ namespace gpsshogi
   {
   private:
     enum { ONE_DIM = 2142 };
-    int index(const Player player, const Position king,
-	      const Ptype ptype, const Position pos) const
+    int index(const Player player, const Square king,
+	      const Ptype ptype, const Square pos) const
     {
       const int x = std::abs(pos.x() - king.x());
       const int y = (king.y() - pos.y()) *
 	(player == osl::BLACK ? 1 : -1) + 8;
       return (ptype - osl::PTYPE_PIECE_MIN) * 17 * 9 + (x * 17 + y);
     }
-    int index(const Player player, const Position king,
+    int index(const Player player, const Square king,
 	      const Piece piece) const
     {
-      return index(player, king, piece.ptype(), piece.position());
+      return index(player, king, piece.ptype(), piece.square());
     }
   public:
     PieceKingRelativeNoSupport() : FeaturesOneNonUniq(ONE_DIM * 2) { }
@@ -1084,14 +1084,14 @@ namespace gpsshogi
       IndexCacheI<MaxActiveWithDuplication> &diffs) const;
   public:
     template <Player KingPlayer>
-    static int index(const Piece p, const Position king) 
+    static int index(const Piece p, const Square king) 
     {
-      return index<KingPlayer>(p.ptypeO(), p.position(), king);
+      return index<KingPlayer>(p.ptypeO(), p.square(), king);
     }
 
     template <Player KingPlayer>
-    static int index(const PtypeO ptypeO, const Position position,
-		     const Position king) 
+    static int index(const PtypeO ptypeO, const Square position,
+		     const Square king) 
     {
       const int king_y = (KingPlayer == BLACK ? king.y() : 10 - king.y());
       const int piece_y = (KingPlayer == BLACK ? position.y() :
@@ -1180,7 +1180,7 @@ namespace gpsshogi
       DIAGONAL,
     };
     int indexY(const Player king,
-	       const Position king_position,
+	       const Square king_position,
 	       PtypeO p1, PtypeO p2,
 	       Direction dir) const
     {
@@ -1197,7 +1197,7 @@ namespace gpsshogi
 	+ (king_y - 1) * 32 * 32 * 3 + X_DIM;
     }
     int indexX(const Player king,
-	       const Position king_position,
+	       const Square king_position,
 	       PtypeO p1, PtypeO p2,
 	       Direction dir) const
     {
@@ -1247,7 +1247,7 @@ namespace gpsshogi
     void countPiece(const NumEffectState &state,
 		    CArray<int, 3> &out) const;
     template <Player Defense>
-    int index(const Position king, int distance0, int count) const
+    int index(const Square king, int distance0, int count) const
     {
       int king_x = (king.x() > 5 ? 10 - king.x() : king.x());
       int king_y = (Defense == WHITE ? 10 - king.y() : king.y());
@@ -1279,7 +1279,7 @@ namespace gpsshogi
     void countPiece(const NumEffectState &state,
 		    CArray<int, 3> &out) const;
     template <Player Defense>
-    int index(const Position king, int count0, int count1, int count2) const
+    int index(const Square king, int count0, int count1, int count2) const
     {
       int king_x = (king.x() > 5 ? 10 - king.x() : king.x());
       int king_y = (Defense == WHITE ? 10 - king.y() : king.y());
@@ -1311,7 +1311,7 @@ namespace gpsshogi
   {
   private:
     template<Player P, Direction Dir>
-    int index(Position king, Position end, int dir) const
+    int index(Square king, Square end, int dir) const
     {
       const int king_x = (king.x() > 5 ? 10 - king.x() : king.x()) - 1;
       const int king_y = (P == BLACK ? king.y() : 10 - king.y()) - 1;
@@ -1336,7 +1336,7 @@ namespace gpsshogi
   {
   private:
     template<Player P>
-    int index(Position king, int mobility) const
+    int index(Square king, int mobility) const
     {
       const int king_x = (king.x() > 5 ? 10 - king.x() : king.x()) - 1;
       const int king_y = (P == BLACK ? king.y() : 10 - king.y()) - 1;
@@ -1377,14 +1377,14 @@ namespace gpsshogi
     template <Player Defense>
     static int effectVertical(const NumEffectState &state, int index)
     {
-      Position king = state.getKingPosition<Defense>();
+      Square king = state.kingSquare<Defense>();
       int result = 0;
       const int x = king.x() + (Defense == BLACK ? -2 + index : 2 - index);
       for (int y_diff = -2; y_diff <= 2; ++y_diff)
       {
 	const int y = king.y() + y_diff;
 	if (x < 1 || x > 9 || y < 1 || y > 9 ||
-	    state.hasEffectBy<PlayerTraits<Defense>::opponent>(Position(x, y)))
+	    state.hasEffectAt<PlayerTraits<Defense>::opponent>(Square(x, y)))
 	{
 	  result |= (1 << (Defense == BLACK ? y_diff + 2 : -y_diff + 2));
 	}
@@ -1425,7 +1425,7 @@ namespace gpsshogi
     size_t maxActive() const { return 16; }
   private:
     template <Player P>
-    static int index(Position king, int effect1, int effect2, int i)
+    static int index(Square king, int effect1, int effect2, int i)
     {
       const int king_y = (P == BLACK ? king.y() : 10 - king.y());
       assert(0 <= effect1 && effect1 < 32);
@@ -1440,14 +1440,14 @@ namespace gpsshogi
     template <Player Defense>
     static int effectVertical(const NumEffectState &state, int index)
     {
-      Position king = state.getKingPosition<Defense>();
+      Square king = state.kingSquare<Defense>();
       int result = 0;
       const int x = king.x() + (Defense == BLACK ? -2 + index : 2 - index);
       for (int y_diff = -2; y_diff <= 2; ++y_diff)
       {
 	const int y = king.y() + y_diff;
 	if (x < 1 || x > 9 || y < 1 || y > 9 ||
-	    state.hasEffectBy<PlayerTraits<Defense>::opponent>(Position(x, y)))
+	    state.hasEffectAt<PlayerTraits<Defense>::opponent>(Square(x, y)))
 	{
 	  result |= (1 << (Defense == BLACK ? y_diff + 2 : -y_diff + 2));
 	}
@@ -1488,7 +1488,7 @@ namespace gpsshogi
     size_t maxActive() const { return 16; }
   private:
     template <Player P>
-    static int index(Position king, int effect1, int effect2, int i, int j)
+    static int index(Square king, int effect1, int effect2, int i, int j)
     {
       const int king_x = (king.x() >= 6 ? 10 - king.x() : king.x());
       if ((P == BLACK && king.x() > 5) ||
@@ -1516,14 +1516,14 @@ namespace gpsshogi
     template <Player Defense>
     static int effectVertical(const NumEffectState &state, int index)
     {
-      Position king = state.getKingPosition<Defense>();
+      Square king = state.kingSquare<Defense>();
       int result = 0;
       const int x = king.x() + (Defense == BLACK ? -2 + index : 2 - index);
       for (int y_diff = -2; y_diff <= 2; ++y_diff)
       {
 	const int y = king.y() + y_diff;
 	if (x < 1 || x > 9 || y < 1 || y > 9 ||
-	    state.hasEffectBy<PlayerTraits<Defense>::opponent>(Position(x, y)))
+	    state.hasEffectAt<PlayerTraits<Defense>::opponent>(Square(x, y)))
 	{
 	  result |= (1 << (Defense == BLACK ? y_diff + 2 : -y_diff + 2));
 	}
@@ -1559,7 +1559,7 @@ namespace gpsshogi
   {
   private:
     template<Player P, Direction Dir>
-    int index(Position king, Position end, int dir) const
+    int index(Square king, Square end, int dir) const
     {
       const int king_x = (king.x() > 5 ? 10 - king.x() : king.x()) - 1;
       const int king_y = (P == BLACK ? king.y() : 10 - king.y()) - 1;
@@ -1615,7 +1615,7 @@ namespace gpsshogi
     template <Player Defense>
     static int effectVertical(const NumEffectState &state, int index)
     {
-      Position king = state.getKingPosition<Defense>();
+      Square king = state.kingSquare<Defense>();
       int result = 0;
       const int x = king.x() + (Defense == BLACK ? -2 + index : 2 - index);
       for (int y_diff = -2; y_diff <= 2; ++y_diff)
@@ -1623,9 +1623,9 @@ namespace gpsshogi
 	const int y = king.y() + y_diff;
 	if (x >= 1 && x <= 9 && y >= 1 && y <= 9)
 	{
-	  const Position pos(x, y);
-	  const Piece piece = state.getPieceAt(pos);
-	  if (!state.hasEffectBy<PlayerTraits<Defense>::opponent>(pos) &&
+	  const Square pos(x, y);
+	  const Piece piece = state.pieceAt(pos);
+	  if (!state.hasEffectAt<PlayerTraits<Defense>::opponent>(pos) &&
 	      (piece.isEmpty() || piece.owner() != Defense))
 	  result |= (1 << (Defense == BLACK ? y_diff + 2 : -y_diff + 2));
 	}
@@ -1665,7 +1665,7 @@ namespace gpsshogi
     size_t maxActive() const { return 8; }
   private:
     template <Player Defense>
-    static int index(Position king, int effect1, int effect2, int i)
+    static int index(Square king, int effect1, int effect2, int i)
     {
       const int king_y = (Defense == BLACK ? king.y() : 10 - king.y());
       assert(0 <= effect1 && effect1 < 32);
@@ -1680,7 +1680,7 @@ namespace gpsshogi
     template <Player Defense>
     static int effectVertical(const NumEffectState &state, int index)
     {
-      Position king = state.getKingPosition<Defense>();
+      Square king = state.kingSquare<Defense>();
       int result = 0;
       const int x = king.x() + (Defense == BLACK ? -2 + index : 2 - index);
       for (int y_diff = -2; y_diff <= 2; ++y_diff)
@@ -1688,9 +1688,9 @@ namespace gpsshogi
 	const int y = king.y() + y_diff;
 	if (x >= 1 && x <= 9 && y >= 1 && y <= 9)
 	{
-	  const Position pos(x, y);
-	  const Piece piece = state.getPieceAt(pos);
-	  if (!state.hasEffectBy<PlayerTraits<Defense>::opponent>(pos) &&
+	  const Square pos(x, y);
+	  const Piece piece = state.pieceAt(pos);
+	  if (!state.hasEffectAt<PlayerTraits<Defense>::opponent>(pos) &&
 	      (piece.isEmpty() || piece.owner() != Defense))
 	  result |= (1 << (Defense == BLACK ? y_diff + 2 : -y_diff + 2));
 	}
@@ -1730,7 +1730,7 @@ namespace gpsshogi
     size_t maxActive() const { return 8; }
   private:
     template <Player Defense>
-    static int index(Position king, int effect1, int effect2, int i)
+    static int index(Square king, int effect1, int effect2, int i)
     {
       const int king_x = (king.x() > 5 ? 10 - king.x() : king.x());
       if ((Defense == BLACK && king.x() > 5) ||
@@ -1753,7 +1753,7 @@ namespace gpsshogi
     template <Player Defense>
     static int effectVertical(const NumEffectState &state, int index)
     {
-      Position king = state.getKingPosition<Defense>();
+      Square king = state.kingSquare<Defense>();
       int result = 0;
       const int x = king.x() + (Defense == BLACK ? -2 + index : 2 - index);
       for (int y_diff = -2; y_diff <= 2; ++y_diff)
@@ -1761,9 +1761,9 @@ namespace gpsshogi
 	const int y = king.y() + y_diff;
 	if (x >= 1 && x <= 9 && y >= 1 && y <= 9)
 	{
-	  const Position pos(x, y);
-	  const Piece piece = state.getPieceAt(pos);
-	  if (!state.hasEffectBy<PlayerTraits<Defense>::opponent>(pos) &&
+	  const Square pos(x, y);
+	  const Piece piece = state.pieceAt(pos);
+	  if (!state.hasEffectAt<PlayerTraits<Defense>::opponent>(pos) &&
 	      (piece.isEmpty() || piece.owner() != Defense))
 	  result |= (1 << (Defense == BLACK ? y_diff + 2 : -y_diff + 2));
 	}
@@ -1823,7 +1823,7 @@ namespace gpsshogi
       int &attacked_count)
     {
       const Player Defense = PlayerTraits<Attack>::opponent;
-      const Position king = state.getKingPosition<Defense>();
+      const Square king = state.kingSquare<Defense>();
       const int min_x = std::max(1, king.x() - 2);
       const int max_x = std::min(9, king.x() + 2);
       const int min_y = std::max(1, king.y() - 2);
@@ -1834,11 +1834,11 @@ namespace gpsshogi
       {
 	for (int x = min_x; x <= max_x; ++x)
 	{
-	  const Position target(x, y);
-	  mask = mask | state.getEffect(target);
+	  const Square target(x, y);
+	  mask = mask | state.effectSetAt(target);
 	}
       }
-      mask = mask & state.getOnBoardMask(Attack);
+      mask = mask & state.piecesOnBoard(Attack);
       const PieceMask promoted_mask = (mask & state.promotedPieces());
       with_knight = (mask & ~state.promotedPieces()).selectBit<KNIGHT>().any();
       mask.clearBit<KNIGHT>();
@@ -1855,17 +1855,17 @@ namespace gpsshogi
       // mask without small pieces is better or mask with small pieces
       // is better?
       PieceMask attacked =
-	state.effectedMask(Attack) & state.getOnBoardMask(Defense);
+	state.effectedMask(Attack) & state.piecesOnBoard(Defense);
       attacked.clearBit<KNIGHT>();
       attacked.clearBit<LANCE>();
       attacked.clearBit<PAWN>();
       PieceMask attacking;
       while (attacked.any())
       {
-	const Piece piece = state.getPieceOf(attacked.takeOneBit());
-	attacking = attacking | state.getEffect(piece.position());
+	const Piece piece = state.pieceOf(attacked.takeOneBit());
+	attacking = attacking | state.effectSetAt(piece.square());
       }
-      attacking = (attacking & state.getOnBoardMask(Attack) & ~mask);
+      attacking = (attacking & state.piecesOnBoard(Attack) & ~mask);
       attacked_count = std::min(5, attacking.countBit());
     }
   };
@@ -1898,7 +1898,7 @@ namespace gpsshogi
       int &attacked_count)
     {
       const Player Defense = PlayerTraits<Attack>::opponent;
-      const Position king = state.getKingPosition<Defense>();
+      const Square king = state.kingSquare<Defense>();
       const int min_x = std::max(1, king.x() - 2);
       const int max_x = std::min(9, king.x() + 2);
       const int min_y = std::max(1, king.y() - 2);
@@ -1909,11 +1909,11 @@ namespace gpsshogi
       {
 	for (int x = min_x; x <= max_x; ++x)
 	{
-	  const Position target(x, y);
-	  mask = mask | state.getEffect(target);
+	  const Square target(x, y);
+	  mask = mask | state.effectSetAt(target);
 	}
       }
-      mask = mask & state.getOnBoardMask(Attack);
+      mask = mask & state.piecesOnBoard(Attack);
       const PieceMask promoted_mask = (mask & state.promotedPieces());
       with_knight = (mask & ~state.promotedPieces()).selectBit<KNIGHT>().any();
       mask.clearBit<KNIGHT>();
@@ -1930,17 +1930,17 @@ namespace gpsshogi
       // mask without small pieces is better or mask with small pieces
       // is better?
       PieceMask attacked =
-	state.effectedMask(Attack) & state.getOnBoardMask(Defense);
+	state.effectedMask(Attack) & state.piecesOnBoard(Defense);
       attacked.clearBit<KNIGHT>();
       attacked.clearBit<LANCE>();
       attacked.clearBit<PAWN>();
       PieceMask attacking;
       while (attacked.any())
       {
-	const Piece piece = state.getPieceOf(attacked.takeOneBit());
-	attacking = attacking | state.getEffect(piece.position());
+	const Piece piece = state.pieceOf(attacked.takeOneBit());
+	attacking = attacking | state.effectSetAt(piece.square());
       }
-      attacking = (attacking & state.getOnBoardMask(Attack) & ~mask);
+      attacking = (attacking & state.piecesOnBoard(Attack) & ~mask);
       attacked_count = std::min(5, attacking.countBit());
     }
   };
@@ -1964,7 +1964,7 @@ namespace gpsshogi
       int &defense_count)
     {
       const Player Defense = PlayerTraits<Attack>::opponent;
-      const Position king = state.getKingPosition<Defense>();
+      const Square king = state.kingSquare<Defense>();
       const int min_x = std::max(1, king.x() - 2);
       const int max_x = std::min(9, king.x() + 2);
       const int min_y = std::max(1, king.y() - 2);
@@ -1975,12 +1975,12 @@ namespace gpsshogi
       {
 	for (int x = min_x; x <= max_x; ++x)
 	{
-	  const Position target(x, y);
-	  mask = mask | state.getEffect(target);
+	  const Square target(x, y);
+	  mask = mask | state.effectSetAt(target);
 	}
       }
-      PieceMask attack_mask = mask & state.getOnBoardMask(Attack);
-      PieceMask defense_mask = mask & state.getOnBoardMask(Defense);
+      PieceMask attack_mask = mask & state.piecesOnBoard(Attack);
+      PieceMask defense_mask = mask & state.piecesOnBoard(Defense);
       const PieceMask attack_promoted_mask = (attack_mask & state.promotedPieces());
       attack_mask.clearBit<KNIGHT>();
       attack_mask.clearBit<LANCE>();
@@ -2017,6 +2017,53 @@ namespace gpsshogi
 			    IndexCacheI<MaxActiveWithDuplication> &) const;
     const std::string name() const { return "King25EffectCountCombinationY"; };
   };
+
+  class BishopExchangeSilverKing : public EvalComponent
+  {
+    static int indexKing(Square king) 
+    {
+      const int y = king.y();
+      if (y >= 3)
+	return -1;
+      return (y-1)*9 + king.x()-1;
+    }
+    static int indexRook(Square rook) 
+    {
+      assert(rook.isOnBoard());
+      const int y = rook.y();
+      if (y >= 6)
+	return -1;
+      return (y-1)*9 + rook.x()-1;
+    }
+    static int indexSilver(Square silver) 
+    {
+      return (silver.y()-1)*9 + silver.x()-1;
+    }
+  public:
+    enum { BISHOP_ONE_DIM = 18 * 81 * (45*2), DIM = BISHOP_ONE_DIM*3 };
+    BishopExchangeSilverKing() : EvalComponent(DIM) { }
+    size_t maxActive() const { return 4; }
+    int eval(const NumEffectState& state) const;
+    void featuresNonUniq(const NumEffectState& state, index_list_t& out, int offset) const;
+    const std::string name() const { return "BishopExchangeSilverKing"; };
+  private:
+    void addOne(Player king_owner, const NumEffectState &state,
+		IndexCacheI<MaxActiveWithDuplication> &diffs, int offset) const;
+  };
+
+  class EnterKingDefense : public EvalComponent
+  {
+  public:
+    enum { DIM = (8+8+8+8)*3 };
+    EnterKingDefense() : EvalComponent(DIM) { }
+    int eval(const NumEffectState& state) const;
+    void featuresNonUniq(const NumEffectState& state, index_list_t& out, int offset) const;
+    const std::string name() const { return "EnterKingDefense"; };
+  private:
+    void addOne(Player king_owner, const NumEffectState &state,
+		IndexCacheI<MaxActiveWithDuplication> &diffs, int offset) const;
+  };
+
 }
 
 
