@@ -116,12 +116,12 @@ namespace gpsshogi
   {
   private:
     template <Player RookOwner>
-    static MultiInt evalOne(const NumEffectState &state, const Position king,
+    static MultiInt evalOne(const NumEffectState &state, const Square king,
 		     const Piece rook, const Offset dir, const MultiWeights&);
-    static int index(Player p, Position pos, Position king, bool horizontal,
+    static int index(Player p, Square pos, Square king, bool horizontal,
 		     bool promoted);
     void featureOne(const NumEffectState &state,
-		    const Position king,
+		    const Square king,
 		    const Piece rook,
 		    const Offset dir,
 		    CArray<int, 612>& feature) const;
@@ -148,12 +148,12 @@ namespace gpsshogi
   class BishopEffect : public EvalComponentMulti
   {
   private:
-    MultiInt evalOne(const NumEffectState &state, const Position king,
+    MultiInt evalOne(const NumEffectState &state, const Square king,
 		     const Piece bishop, const Direction dir, const MultiWeights&) const;
-    int index(Player p, Position pos, Position king, bool ur,
+    int index(Player p, Square pos, Square king, bool ur,
 	      bool promoted) const;
     void featureOne(const NumEffectState &state,
-		    const Position king,
+		    const Square king,
 		    const Piece bishop,
 		    const Direction dir,
 		    CArray<int, 612>& feature) const;
@@ -183,7 +183,7 @@ namespace gpsshogi
   {
   public:
     static PtypeO getPtypeO(const NumEffectState &state,
-			    const Position pos, const Player player,
+			    const Square pos, const Player player,
 			    const Direction dir);
     static void addPtypeO(const Piece rook,
 			  const PtypeO ptypeO,
@@ -226,11 +226,11 @@ namespace gpsshogi
   class MajorEffectPieceKingRelative : public FeaturesOneNonUniq
   {
   private:
-    static int index(Player p, Position pos, Position king,
+    static int index(Player p, Square pos, Square king,
 		     PtypeO ptypeO, bool attack,
 		     bool horizontal, bool promoted);
     static void addOne(const NumEffectState &state,
-		       const Position pos, const Player player,
+		       const Square pos, const Player player,
 		       const Direction dir, const bool promoted,
 		       IndexCacheI<MaxActiveWithDuplication> &features);
   public:
@@ -378,12 +378,12 @@ namespace gpsshogi
     size_t maxActive() const { return 24; }
   private:
     template <Player King>
-    int index(const Position king, const Piece rook, const Piece bishop) const
+    int index(const Square king, const Piece rook, const Piece bishop) const
     {
-      const int rook_x = std::abs(king.x() - rook.position().x());
-      const int bishop_x = std::abs(king.x() - bishop.position().x());
-      const int rook_y = (King == BLACK ? rook.position().y() - king.y() : king.y() - rook.position().y());
-      const int bishop_y = (King == BLACK ? bishop.position().y() - king.y() : king.y() - bishop.position().y());
+      const int rook_x = std::abs(king.x() - rook.square().x());
+      const int bishop_x = std::abs(king.x() - bishop.square().x());
+      const int rook_y = (King == BLACK ? rook.square().y() - king.y() : king.y() - rook.square().y());
+      const int bishop_y = (King == BLACK ? bishop.square().y() - king.y() : king.y() - bishop.square().y());
       return bishop_y + 8 + 17 * (bishop_x + 9 * (rook_y + 8 + 17 * (rook_x + 9 * ((bishop.owner() == King ? 1 : 0) + 2 * ((rook.owner() == King ? 1 : 0) + 2 * (2 * (bishop.isPromoted() ? 1 : 0) + (rook.isPromoted() ? 1 : 0)))))));
     }
   };
@@ -401,7 +401,7 @@ namespace gpsshogi
     const std::string name() const { return "NumPiecesBetweenBishopAndKing"; }
     size_t maxActive() const { return 2; }
     static int countBetween(const NumEffectState &state,
-			    Position king, Piece bishop);
+			    Square king, Piece bishop);
     void showAllOne(const Weights&,
 		    int n,
 		    std::ostream &os) const;
@@ -445,7 +445,7 @@ namespace gpsshogi
     const std::string name() const { return "NumPiecesBetweenBishopAndKingCombination"; }
     size_t maxActive() const { return 2; }
     static void countBetween(const NumEffectState &state,
-			     Position king, Piece bishop,
+			     Square king, Piece bishop,
 			     int &self, int &opp);
   };
 
@@ -524,6 +524,82 @@ namespace gpsshogi
     const std::string name() const { return "BishopStandRank5"; }
     size_t maxActive() const { return 2; }
   };
+
+  class MajorCheckWithCapture : public FeaturesOneNonUniq
+  {
+    enum {
+      ONE_DIM = PTYPE_SIZE * 2/*bishop or rook*/ * 2 /*promotable*/
+    };
+  public:
+    MajorCheckWithCapture() : FeaturesOneNonUniq(ONE_DIM) { }
+    void featuresOneNonUniq(const NumEffectState &state,
+			    IndexCacheI<MaxActiveWithDuplication> &) const;
+    void addOne(Player owner, const NumEffectState &state,
+		IndexCacheI<MaxActiveWithDuplication> &) const;
+    const std::string name() const { return "MajorCheckWithCapture"; }
+    size_t maxActive() const { return 8; }
+    static size_t index(Ptype ptype, bool is_rook, bool can_promote) 
+    {
+      return ptype * 4 + is_rook * 2 + can_promote;
+    }
+  };
+
+  class RookSilverKnight : public FeaturesOneNonUniq
+  {
+    enum {
+      ONE_DIM = 5 * 9 * 9 * 9 * 9 * 9
+    };
+  public:
+    RookSilverKnight() : FeaturesOneNonUniq(ONE_DIM) { }
+    void featuresOneNonUniq(const NumEffectState &state,
+			    IndexCacheI<MaxActiveWithDuplication> &) const;
+    const std::string name() const { return "RookSilverKnight"; }
+    size_t maxActive() const { return 2 * 4 * 4; }
+  private:
+    static size_t index(int rook_x, int rook_y, int silver_x, int silver_y,
+			int knight_x, int knight_y)
+    {
+      return knight_y + 9 * (knight_x + 9 * (silver_y + 9 * (silver_x + 9 * (rook_y + 9 * rook_x))));
+    }
+  };
+
+  class BishopSilverKnight : public FeaturesOneNonUniq
+  {
+    enum {
+      ONE_DIM = 5 * 9 * 9 * 9 * 9 * 9
+    };
+  public:
+    BishopSilverKnight() : FeaturesOneNonUniq(ONE_DIM) { }
+    void featuresOneNonUniq(const NumEffectState &state,
+			    IndexCacheI<MaxActiveWithDuplication> &) const;
+    const std::string name() const { return "BishopSilverKnight"; }
+    size_t maxActive() const { return 2 * 4 * 4; }
+  private:
+    static size_t index(int bishop_x, int bishop_y, int silver_x, int silver_y,
+			int knight_x, int knight_y)
+    {
+      return knight_y + 9 * (knight_x + 9 * (silver_y + 9 * (silver_x + 9 * (bishop_y + 9 * bishop_x))));
+    }
+  };
+
+  class AttackMajorsInBase : public FeaturesOneNonUniq
+  {
+    enum {
+      ONE_DIM = PTYPE_SIZE * PTYPE_SIZE * 2 * 2 * 2
+    };
+  public:
+    AttackMajorsInBase() : FeaturesOneNonUniq(ONE_DIM) { }
+    void featuresOneNonUniq(const NumEffectState &state,
+			    IndexCacheI<MaxActiveWithDuplication> &) const;
+    const std::string name() const { return "AttackMajorsInBase"; }
+    size_t maxActive() const { return 4; }
+    static int index(Ptype support, Ptype attack, bool has_gold, 
+		     bool rook_support, bool bishop_support)
+    {
+      return (unpromoteSafe(support)*16 + unpromoteSafe(attack))*8+has_gold*4
+	+rook_support*2+bishop_support;
+    }
+  };
 }
 
 #endif /* _GPSSSHOGI_SAMPLE_EVAL_LIB_MAJORPIECE_H */
