@@ -12,7 +12,7 @@ namespace gpsshogi
   private:
     bool canRetreat(const osl::state::NumEffectState &state,
 		    const osl::Piece silver) const;
-    int index(Player P, Position pos) const
+    int index(Player P, Square pos) const
     {
       return (P == BLACK ? (pos.y() - 1) : (9 - pos.y()));
     }
@@ -33,7 +33,7 @@ namespace gpsshogi
   private:
     bool canRetreat(const osl::state::NumEffectState &state,
 		    const osl::Piece gold) const;
-    int index(Player P, Position pos) const
+    int index(Player P, Square pos) const
     {
       return (P == BLACK ? (pos.y() - 1) : (9 - pos.y()));
     }
@@ -54,11 +54,11 @@ namespace gpsshogi
   private:
     bool canMove(const osl::state::NumEffectState &state,
 		 const osl::Piece gold) const;
-    int indexX(Position pos) const
+    int indexX(Square pos) const
     {
       return (pos.x() > 5 ? 9 - pos.x() : pos.x() - 1);
     }
-    int indexY(Player P, Position pos) const
+    int indexY(Player P, Square pos) const
     {
       return (P == BLACK ? (pos.y() - 1) : (9 - pos.y())) + 5;
     }
@@ -95,8 +95,8 @@ namespace gpsshogi
     void featuresOneNonUniq(const NumEffectState &state,
 			    IndexCacheI<MaxActiveWithDuplication> &) const;
     const std::string name() const { return "BishopRookFork"; };
-    static bool isBishopForkPosition(const NumEffectState& state, Player defense, const Position a, const Position b);
-    static bool isRookForkPosition(const NumEffectState& state, Player defense, const Position a, const Position b);
+    static bool isBishopForkSquare(const NumEffectState& state, Player defense, const Square a, const Square b);
+    static bool isRookForkSquare(const NumEffectState& state, Player defense, const Square a, const Square b);
     static int bishopIndex(Ptype a, Ptype b)
     {
       if (a > b)
@@ -109,10 +109,10 @@ namespace gpsshogi
     }
   private:
     static bool findDropInLine(const NumEffectState& state, Player defense, 
-			       const Position a, const Position b, Piece king);
+			       const Square a, const Square b, Piece king);
     static bool testCenter(const NumEffectState& state, Player defense, 
-			   const Position a, const Position b, Piece king,
-			   Position center);
+			   const Square a, const Square b, Piece king,
+			   Square center);
   };
 
   class KnightFork : public FeaturesOneNonUniq
@@ -123,7 +123,7 @@ namespace gpsshogi
     void featuresOneNonUniq(const NumEffectState &state,
 			    IndexCacheI<MaxActiveWithDuplication> &) const;
     const std::string name() const { return "KnightFork"; };
-    static bool isForkPosition(const NumEffectState& state, Player defense, 
+    static bool isForkSquare(const NumEffectState& state, Player defense, 
 			       int y, int x0, int x1);
     static int index(Ptype a, Ptype b)
     {
@@ -137,11 +137,11 @@ namespace gpsshogi
   {
     bool cantAdvance(const NumEffectState &state, const Piece pawn) const
     {
-      return cantAdvance(state, pawn.ptypeO(), pawn.position());
+      return cantAdvance(state, pawn.ptypeO(), pawn.square());
     }
     bool cantAdvance(const NumEffectState &state,
-		     const PtypeO ptypeO, const Position position) const;
-    int index(Player P, Position pos) const
+		     const PtypeO ptypeO, const Square position) const;
+    int index(Player P, Square pos) const
     {
       return (P == BLACK ? (pos.y() - 1) : (9 - pos.y()));
     }
@@ -164,10 +164,10 @@ namespace gpsshogi
   {
     bool cantAdvance(const NumEffectState &state, const Piece pawn) const
     {
-      return cantAdvance(state, pawn.ptypeO(), pawn.position());
+      return cantAdvance(state, pawn.ptypeO(), pawn.square());
     }
     bool cantAdvance(const NumEffectState &state,
-		     const PtypeO ptypeO, const Position position) const;
+		     const PtypeO ptypeO, const Square position) const;
     enum { DIM = 9 };
   public:
     KnightAdvance() : EvalComponentMulti(DIM) { }
@@ -233,7 +233,7 @@ namespace gpsshogi
   private:
     int index(const Piece king, int x, bool attack) const
     {
-      const int king_x = king.position().x();
+      const int king_x = king.square().x();
       const int target_x = (king_x > 5 ? 10 - king_x : king_x);
       if (king_x >= 6 || (king.owner() == WHITE && king_x == 5))
 	x = 10 - x;
@@ -248,8 +248,8 @@ namespace gpsshogi
     int index(const Piece king, int x) const
     {
       const int king_y = (king.owner() == BLACK ?
-			  king.position().y() : 10 - king.position().y());
-      return std::abs(x - king.position().x()) * 9 + king_y - 1;
+			  king.square().y() : 10 - king.square().y());
+      return std::abs(x - king.square().x()) * 9 + king_y - 1;
     }
   public:
     PawnDropY(bool defense=true) : EvalComponentMulti(81), is_defense(defense) { }
@@ -292,7 +292,7 @@ namespace gpsshogi
     const std::string name() const { return "PawnDropPawnStand"; };
   private:
     template <bool Attack>
-    int index(const Position king, int x) const
+    int index(const Square king, int x) const
     {
       return std::abs(king.x() - x) + (Attack ? 0 : 9);
     }
@@ -311,7 +311,7 @@ namespace gpsshogi
     template <bool Attack>
     int index(const Piece king, int x) const
     {
-      const int king_x = king.position().x();
+      const int king_x = king.square().x();
       const int target_x = (king_x > 5 ? 10 - king_x : king_x);
       if (king_x >= 6 || (king.owner() == WHITE && king_x == 5))
 	x = 10 - x;
@@ -333,8 +333,8 @@ namespace gpsshogi
     int index(const Piece king, int x) const
     {
       const int king_y = (king.owner() == BLACK ?
-			  king.position().y() : 10 - king.position().y());
-      return std::abs(x - king.position().x()) * 9 + king_y - 1 +
+			  king.square().y() : 10 - king.square().y());
+      return std::abs(x - king.square().x()) * 9 + king_y - 1 +
 	(Attack ? 0 : 81);
     }
   };
@@ -477,9 +477,9 @@ namespace gpsshogi
   private:
     int index(const Piece piece) const
     {
-      return index(piece.owner(), piece.ptype(), piece.position());
+      return index(piece.owner(), piece.ptype(), piece.square());
     }
-    int index(const Player, const Ptype ptype, const Position pos) const;
+    int index(const Player, const Ptype ptype, const Square pos) const;
   public:
     PtypeY() : EvalComponentMulti(9 * PTYPE_SIZE) { }
     MultiInt eval(const osl::state::NumEffectState &state, const MultiWeights& weights,
@@ -498,9 +498,9 @@ namespace gpsshogi
   private:
     int index(const Piece piece) const
     {
-      return index(piece.owner(), piece.ptype(), piece.position());
+      return index(piece.owner(), piece.ptype(), piece.square());
     }
-    int index(const Player, const Ptype ptype, const Position pos) const;
+    int index(const Player, const Ptype ptype, const Square pos) const;
   public:
     PtypeX() : EvalComponentMulti(5 * PTYPE_SIZE) { }
     MultiInt eval(const osl::state::NumEffectState &state, const MultiWeights& weights,
@@ -545,7 +545,7 @@ namespace gpsshogi
     void featuresX(const NumEffectState &state,
 		   const PieceMask promoted,
 		   IndexCacheI<MaxActiveWithDuplication> &features) const;
-    int index(bool attack, const Position king,
+    int index(bool attack, const Square king,
 	      const Player owner, int x_diff) const
     {
       const int y = (owner == BLACK ? king.y() : 10 - king.y());
@@ -576,7 +576,7 @@ namespace gpsshogi
   class LanceEffectPieceKingRelative : public FeaturesOneNonUniq
   {
   private:
-    int index(Player p, Position pos, Position king,
+    int index(Player p, Square pos, Square king,
 	      PtypeO ptypeO, bool attack) const
     {
       const int y_diff = (p == BLACK ? king.y() - pos.y() : pos.y() - king.y());
@@ -590,7 +590,7 @@ namespace gpsshogi
     }
 
     void addOne(const NumEffectState &state,
-		const Position pos, const Player player,
+		const Square pos, const Player player,
 		IndexCacheI<MaxActiveWithDuplication> &features) const;
   public:
     LanceEffectPieceKingRelative()
@@ -644,7 +644,7 @@ namespace gpsshogi
 			    IndexCacheI<MaxActiveWithDuplication> &) const;
   private:
     int index(const Player player,
-	      const Position king, const Position pawn,
+	      const Square king, const Square pawn,
 	      const bool has_pawn_effect,
 	      PieceMask effect_mask) const
     {
@@ -691,6 +691,16 @@ namespace gpsshogi
 			    2 * support))));
     }
   };
+
+  class SilverAdvance26 : public FeaturesOneNonUniq
+  {
+    enum { DIM = 1 };
+  public:
+    SilverAdvance26() : FeaturesOneNonUniq(DIM) { }
+    void featuresOneNonUniq
+    (const NumEffectState &state, IndexCacheI<MaxActiveWithDuplication> &features) const;
+    const std::string name() const { return "SilverAdvance26"; };
+  };
 }
 
 #endif /* _GPSSSHOGI_SAMPLE_EVAL_LIB_MINORPIECE_H */
