@@ -16,8 +16,12 @@
 #include "eval/ptypeEval.h"
 #include "eval/piecePairKing.h"
 #include "osl/progress/ml/newProgress.h"
+#include "osl/eval/ml/openMidEndingEval.h"
 #include "osl/stl/stack.h"
+#include "osl/misc/binaryIO.h"
+#include "osl/oslConfig.h"
 
+#include <boost/filesystem.hpp>
 #include <iomanip>
 #include <fstream>
 #include <iostream>
@@ -129,26 +133,7 @@ public:
     assert((int)eval->flat.size() <= OthersLimit);
     assert((int)eval->legacy.size() <= OthersLimit);
     assert((int)eval->triple.size() <= OthersLimit);
-    Entry e;
-    e.sum = 0;
-    e.progress = progress::ml::NewProgress(state);
-    const int progress = eval->progress(e.progress.progress());
-    for (size_t i=0; i<eval->flat.size(); ++i)
-      e.flat[i] = eval->flat[i].eval(state);
-    MultiInt multi_sum;
-    for (size_t i=0; i<eval->legacy.size(); ++i) {
-      e.legacy[i] = eval->legacy[i].eval(state, eval->legacy_weights[i], e.legacy_state[i]);
-      multi_sum += e.legacy[i];
-    }
-    for (size_t i=0; i<eval->triple.size(); ++i) {
-      e.triple[i] = eval->triple[i].evalMulti(state, e.triple_state[i]);
-      multi_sum += e.triple[i];
-    }
-    // sum
-    for (size_t i=0; i<eval->flat.size(); ++i)
-      e.sum += e.flat[i];
-    e.sum = eval->compose(e.sum, multi_sum, progress);
-    values.push(e);
+    reset(state);
   }
   void push(const NumEffectState& new_state, Move moved)
   {
@@ -180,6 +165,35 @@ public:
   }
   void pop() { assert(! values.empty()); values.pop(); }
   int value() const { assert(! values.empty()); return values.top().sum; }
+  Progress16 progress16() const {
+    assert(! values.empty());
+    return values.top().progress.progress16();
+  }
+  void reset(const NumEffectState& state)
+  {
+    while(! values.empty())
+      pop();
+    Entry e;
+    e.sum = 0;
+    e.progress = progress::ml::NewProgress(state);
+    const int progress = eval->progress(e.progress.progress());
+    for (size_t i=0; i<eval->flat.size(); ++i)
+      e.flat[i] = eval->flat[i].eval(state);
+    MultiInt multi_sum;
+    for (size_t i=0; i<eval->legacy.size(); ++i) {
+      e.legacy[i] = eval->legacy[i].eval(state, eval->legacy_weights[i], e.legacy_state[i]);
+      multi_sum += e.legacy[i];
+    }
+    for (size_t i=0; i<eval->triple.size(); ++i) {
+      e.triple[i] = eval->triple[i].evalMulti(state, e.triple_state[i]);
+      multi_sum += e.triple[i];
+    }
+    // sum
+    for (size_t i=0; i<eval->flat.size(); ++i)
+      e.sum += e.flat[i];
+    e.sum = eval->compose(e.sum, multi_sum, progress);
+    values.push(e);
+  }
 };
 
 gpsshogi::EvalValueStack * gpsshogi::
@@ -348,9 +362,7 @@ OpenMidEndingBase::save(const char *filename) const
     for (size_t j=0; j<triple[i].dimension(); ++j)
       os << triple[i].value(j) << std::endl;
   }
-  std::string f = filename;
-  std::cerr << f << " " << f.find(".txt") << " " << f.substr(0, f.find(".txt")) + "-info.txt" << std::endl;
-  
+  std::string f = filename;  
   if (f.find(".txt") != f.npos) {
     f = f.substr(0, f.find(".txt")) + "-info.txt";
     std::ofstream os(f.c_str());
@@ -388,26 +400,32 @@ OpenMidEndingBase::save(const char *filename) const
 void gpsshogi::
 OpenMidEndingBase::setWeight(const double *w)
 {
-  double pawn = 1.0;
+  double scale = 1.0;
   // assuming we have PieceEval
-  pawn = 128.0/w[PAWN];
+  scale = 128.0/w[PAWN];
+  setWeightScale(w, scale);
+}
+
+void gpsshogi::
+OpenMidEndingBase::setWeightScale(const double *w, double scale)
+{
   int dim = 0;
   for (size_t i=0; i<flat.size(); ++i) {
-    flat[i].setWeightScale(&w[0]+dim, pawn);
+    flat[i].setWeightScale(&w[0]+dim, scale);
     dim += flat[i].dimension();
   }
   for (size_t i=0; i<legacy.size(); ++i) {
     for (size_t j=0; j<legacy[i].dimensionOne(); ++j) {
       MultiInt value;
       for (size_t s=0; s<value.size(); ++s)
-	value[s] = (int)round(w[legacy_one_dim*s + dim + j] * pawn);
+	value[s] = (int)round(w[legacy_one_dim*s + dim + j] * scale);
       legacy_weights[i].setValue(j, value);
     }
     dim += legacy[i].dimensionOne();
   }
   dim += legacy_one_dim*(MultiInt::size()-1);
   for (size_t i=0; i<triple.size(); ++i) {
-    triple[i].setWeightScale(&w[0]+dim, pawn);
+    triple[i].setWeightScale(&w[0]+dim, scale);
     dim += triple[i].dimension();
   }
 }
@@ -483,22 +501,25 @@ OpenMidEndingBase::features(const NumEffectState& state, double& value,
       {
 	out.push_back(std::make_pair(his_features[j].first + offset + dim,
 				     his_features[j].second * w2 * (c0 - progress)));
-	out.push_back(std::make_pair(his_features[j].first + offset + dim + legacy_one_dim,
-				     his_features[j].second * w2 * progress));
+	if (progress > 0)
+	  out.push_back(std::make_pair(his_features[j].first + offset + dim + legacy_one_dim,
+				       his_features[j].second * w2 * progress));
       }
       else if (progress < c1) 
       {
 	out.push_back(std::make_pair(his_features[j].first + offset + dim + legacy_one_dim,
 				     his_features[j].second * w2 * (c1 - progress)));
-	out.push_back(std::make_pair(his_features[j].first + offset + dim + legacy_one_dim*2,
-				     his_features[j].second * w2 * (progress - c0)));
+	if (progress > c0)
+	  out.push_back(std::make_pair(his_features[j].first + offset + dim + legacy_one_dim*2,
+				       his_features[j].second * w2 * (progress - c0)));
       }
       else 
       {
 	out.push_back(std::make_pair(his_features[j].first + offset + dim + legacy_one_dim*2,
 				     his_features[j].second * c0 * (progress_max - progress)));
-	out.push_back(std::make_pair(his_features[j].first + offset + dim + legacy_one_dim*3,
-				     his_features[j].second * c0 * (progress - c1)));
+	if (progress > c1)
+	  out.push_back(std::make_pair(his_features[j].first + offset + dim + legacy_one_dim*3,
+				       his_features[j].second * c0 * (progress - c1)));
       }
 #else
       if (progress < c) 
@@ -530,6 +551,11 @@ OpenMidEndingBase::features(const NumEffectState& state, double& value,
   }
   value = compose(value, multi_sum, progress);
   assert(out.size() <= maxActive());
+#ifdef EVAL_QUAD
+  if (c0 == w2)
+    for (size_t i = 0; i < out.size(); ++i) 
+      out[i].second /= c0;
+#endif
 }
 
 void gpsshogi::
@@ -599,8 +625,8 @@ OpenMidEndingBase::showEvalSummary(const NumEffectState& state) const
 {
   for (int y=1; y<=9; ++y) {
     for (int x=9; x>=1; --x) {
-      const Position position(x,y);
-      const Piece piece = state.getPieceOnBoard(position);
+      const Square position(x,y);
+      const Piece piece = state.pieceOnBoard(position);
       if (piece.isPiece())
 	std::cerr << std::setw(4) << pieceValue(state, piece);
       else
@@ -646,6 +672,8 @@ KOpenMidEnding() : OpenMidEndingBase(true)
   flat.push_back(new King25EffectAttack);
   flat.push_back(new King25EffectYAttack);
   flat.push_back(new PiecePairKingFlat);
+  flat.push_back(new BishopExchangeSilverKing);
+  flat.push_back(new EnterKingDefense);
   // 
   legacy.push_back(new PieceStand);
   legacy.push_back(new King25EffectEach);
@@ -769,6 +797,11 @@ KOpenMidEnding() : OpenMidEndingBase(true)
   triple.push_back(new EvalComponentStages(new BishopStandRank5));
   triple.push_back(new EvalComponentStages(new KnightFork));
   triple.push_back(new EvalComponentStages(new NonPawnAttackedPtypePair));
+  triple.push_back(new EvalComponentStages(new MajorCheckWithCapture));
+  triple.push_back(new EvalComponentStages(new SilverAdvance26));
+  triple.push_back(new EvalComponentStages(new RookSilverKnight));
+  triple.push_back(new EvalComponentStages(new BishopSilverKnight));
+  triple.push_back(new EvalComponentStages(new AttackMajorsInBase));
   addFinished();
 }
 
@@ -787,6 +820,8 @@ StableOpenMidEnding() : OpenMidEndingBase(false)
   flat.push_back(new King25EffectAttack);
   flat.push_back(new King25EffectYAttack);
   flat.push_back(new PiecePairKingFlat);
+  flat.push_back(new BishopExchangeSilverKing);
+  flat.push_back(new EnterKingDefense);
   // 
   legacy.push_back(new PieceStand);
   legacy.push_back(new King25EffectEach);
@@ -910,6 +945,11 @@ StableOpenMidEnding() : OpenMidEndingBase(false)
   triple.push_back(new EvalComponentStages(new BishopStandRank5));
   triple.push_back(new EvalComponentStages(new KnightFork));
   triple.push_back(new EvalComponentStages(new NonPawnAttackedPtypePair));
+  triple.push_back(new EvalComponentStages(new MajorCheckWithCapture));
+  triple.push_back(new EvalComponentStages(new SilverAdvance26));
+  triple.push_back(new EvalComponentStages(new RookSilverKnight));
+  triple.push_back(new EvalComponentStages(new BishopSilverKnight));
+  triple.push_back(new EvalComponentStages(new AttackMajorsInBase));
   addFinished();
 }
 
@@ -918,6 +958,258 @@ gpsshogi::StableOpenMidEnding::
 {
 }
 
+/* ------------------------------------------------------------------------- */
+
+gpsshogi::
+OslOpenMidEnding::OslOpenMidEnding()
+  : stable_eval(new StableOpenMidEnding), values(new int[dimension()])
+{
+  static bool initialized = false;
+  if (! initialized) {
+    initialized = true;
+    osl::eval::ml::OpenMidEndingEval::setUp();
+  }
+  std::fill(&values[0], &values[0]+dimension(), 0);
+  std::string filename = OslConfig::home();
+  filename += "/data/eval.bin";
+
+  if (boost::filesystem::exists(filename.c_str())) {
+    std::cerr << "loading " << filename << "\n";
+    std::ifstream is(filename.c_str());
+    vector<double> w(dimension());
+    typedef osl::misc::BinaryElementReader<int> reader_t;
+    reader_t reader(is);
+    for (size_t i=0; i<dimension(); ++i) {
+      if (! reader.hasNext()) {
+	std::cerr << filename << " read failed " << i << "\n";
+	break;
+      }
+      w[i] = values[i] = reader.read();
+    }
+    stable_eval->setWeightScale(&w[0], 1.0);
+
+    for (size_t i=0; i<dimension(); ++i) {
+      if (findFeature(i).get<0>() == "PiecePair")
+	values[i] = stable_eval->flatValue(i);
+      assert(values[i] == stable_eval->flatValue(i));
+    }
+  }  
+}
+
+gpsshogi::
+OslOpenMidEnding::~OslOpenMidEnding()
+{
+}
+
+int gpsshogi::OslOpenMidEnding::progressMax() const
+{
+  return progress::ml::NewProgress::maxProgress();
+}
+
+
+int gpsshogi::OslOpenMidEnding::progress(const NumEffectState &state) const
+{
+  progress::ml::NewProgress progress(state);
+  return this->progress(progress.progress());
+}
+
+int gpsshogi::OslOpenMidEnding::progress(int progress_value) const
+{
+  return progress_value;
+}
+
+size_t gpsshogi::
+OslOpenMidEnding::maxActive() const
+{
+  return stable_eval->maxActive();
+}
+
+class gpsshogi::OslOpenMidEnding::Stack : public EvalValueStack
+{
+  struct Entry
+  {
+    osl::eval::ml::OpenMidEndingEval osl_eval;
+    Entry() : osl_eval(NumEffectState()) {}
+  };
+  osl::stack<Entry> values;
+public:  
+  Stack(OslOpenMidEnding *ev, const NumEffectState& state) { 
+    reset(state);
+  }
+  void push(const NumEffectState& new_state, Move moved)
+  {
+    Entry e = values.top();
+    e.osl_eval.update(new_state, moved);
+    values.push(e);
+  }
+  void pop() { assert(! values.empty()); values.pop(); }
+  int value() const {
+    assert(! values.empty()); return values.top().osl_eval.value(); 
+  }
+  Progress16 progress16() const {
+    assert(! values.empty());
+    return values.top().osl_eval.progress16();
+  }
+  void reset(const NumEffectState& state)
+  {
+    while(! values.empty())
+      pop();
+    Entry e;
+    e.osl_eval = osl::eval::ml::OpenMidEndingEval(state);
+    values.push(e);
+  }
+};
+
+gpsshogi::EvalValueStack * gpsshogi::
+OslOpenMidEnding::newStack(const NumEffectState& state)
+{
+  return new Stack(this, state);
+}
+
+int gpsshogi::
+OslOpenMidEnding::pawnValue() const
+{
+  const int scale = progressMax();
+  return 128 * scale;
+}
+
+int gpsshogi::
+OslOpenMidEnding::openingValue(const NumEffectState& state) const
+{
+  osl::eval::ml::OpenMidEndingEval osl_eval(state);
+  return osl_eval.openingValue();
+}
+int gpsshogi::
+OslOpenMidEnding::midgameValue(const NumEffectState& state) const
+{
+  osl::eval::ml::OpenMidEndingEval osl_eval(state);
+  return osl_eval.midgameValue();
+}
+#ifdef EVAL_QUAD
+int gpsshogi::
+OslOpenMidEnding::midgame2Value(const NumEffectState& state) const
+{
+  osl::eval::ml::OpenMidEndingEval osl_eval(state);
+  return osl_eval.midgame2Value();
+}
+#endif
+int gpsshogi::
+OslOpenMidEnding::endgameValue(const NumEffectState& state) const
+{
+  osl::eval::ml::OpenMidEndingEval osl_eval(state);
+  return osl_eval.endgameValue();
+}
+int gpsshogi::
+OslOpenMidEnding::progressIndependentValue(const NumEffectState& state) const
+{
+  osl::eval::ml::OpenMidEndingEval osl_eval(state);
+  return osl_eval.progressIndependentValue();
+}
+
+int gpsshogi::
+OslOpenMidEnding::eval(const NumEffectState& state) const
+{
+  osl::eval::ml::OpenMidEndingEval osl_eval(state, false);
+  return osl_eval.value();
+}
+
+size_t gpsshogi::
+OslOpenMidEnding::dimension() const
+{
+  return stable_eval->dimension();
+}
+int gpsshogi::
+OslOpenMidEnding::flatValue(size_t index) const 
+{
+  if (index >= dimension())
+    std::cerr << "flatValue " << index << ' ' << dimension() << "\n";
+  assert(index < dimension());
+  return values[index];
+}
+
+bool gpsshogi::
+OslOpenMidEnding::load(const char *filename)
+{
+  std::ifstream is(filename);
+  boost::scoped_array<double> values(new double[dimension()]);
+  for (size_t i=0; i<dimension(); ++i)
+    is >> values[i];
+  setWeight(&values[0]);
+  return is;
+}
+
+void gpsshogi::
+OslOpenMidEnding::save(const char *filename) const
+{
+  stable_eval->save(filename);	// for "-info.txt"
+  std::ofstream os(filename);
+  for (size_t i=0; i<dimension(); ++i)
+    os << values[i] << "\n";
+  std::string f = filename;  
+}
+
+void gpsshogi::
+OslOpenMidEnding::setWeight(const double *w)
+{
+  double scale = 1.0;
+  // assuming we have PieceEval
+  scale = 128.0/w[PAWN];
+  setWeightScale(w, scale);
+}
+
+void gpsshogi::
+OslOpenMidEnding::setWeightScale(const double *w, double scale)
+{
+  for (size_t i=0; i<dimension(); ++i)
+    values[i] = round(w[i]*scale);
+  osl::eval::ml::OpenMidEndingEval::resetWeights(&values[0], dimension());
+  stable_eval->setWeightScale(w, scale);
+}
+
+void gpsshogi::
+OslOpenMidEnding::saveWeight(double *w) const
+{
+  std::copy(&values[0], &values[0]+dimension(), w);
+}
+
+void gpsshogi::
+OslOpenMidEnding::features(const NumEffectState& state, double& value,
+			   vector<std::pair<int, double> >& out,
+			   int offset) const
+{
+  assert(stable_eval->progressMax() == progressMax());
+  stable_eval->features(state, value, out, offset);
+}
+
+void gpsshogi::
+OslOpenMidEnding::showSummary(std::ostream& os) const
+{
+  stable_eval->showSummary(os);
+}
+
+void gpsshogi::
+OslOpenMidEnding::showAll(std::ostream& os) const
+{
+  stable_eval->showAll(os);
+}
+
+void gpsshogi::
+OslOpenMidEnding::setRandom()
+{
+}
+
+bool gpsshogi::
+OslOpenMidEnding::hasPieceValue() const
+{
+  return false;
+}
+
+boost::tuple<std::string, int, int> gpsshogi::
+OslOpenMidEnding::findFeature(size_t index) const
+{
+  return stable_eval->findFeature(index);
+}
+
 /* ------------------------------------------------------------------------- */
 // ;;; Local Variables:
 // ;;; mode:c++
