@@ -23,7 +23,7 @@ namespace gpsshogi
     void showSummary(std::ostream&) const;
     void showAll(std::ostream& os) const;
     void setRandom();
-    void setWeightScale(const double*, const double& scale);
+    void setWeightScale(const double*, double);
     const std::string name() const { return "PiecePair"; };
   };
 }
