@@ -69,19 +69,7 @@ class gpsshogi::ProgressFeatureBase::Stack : public EvalValueStack
 public:  
   Stack(ProgressFeatureBase *ev, const NumEffectState& state) : eval(ev) { 
     assert((int)eval->all.size() <= OthersLimit);
-    Entry e(state);
-    int progress = 0;
-    for (size_t i=0; i<eval->all.size(); ++i) {
-      e.all[i] = eval->all[i].eval(state);
-      progress += e.all[i];
-    }
-    int max = eval->maxProgress();
-    e.eval_value = compose(progress, max,
-			   e.test_eval.progressIndependentValue(),
-			   e.test_eval.openingValue(),
-			   e.test_eval.midgameValue(),
-			   e.test_eval.endgameValue());
-    values.push(e);
+    reset(state);
   }
   void push(const NumEffectState& new_state, Move moved)
   {
@@ -103,6 +91,24 @@ public:
   }
   void pop() { assert(! values.empty()); values.pop(); }
   int value() const { assert(! values.empty()); return values.top().eval_value; }
+  void reset(const NumEffectState& state)
+  {
+    while (! values.empty())
+      pop();
+    Entry e(state);
+    int progress = 0;
+    for (size_t i=0; i<eval->all.size(); ++i) {
+      e.all[i] = eval->all[i].eval(state);
+      progress += e.all[i];
+    }
+    int max = eval->maxProgress();
+    e.eval_value = compose(progress, max,
+			   e.test_eval.progressIndependentValue(),
+			   e.test_eval.openingValue(),
+			   e.test_eval.midgameValue(),
+			   e.test_eval.endgameValue());
+    values.push(e);
+  }
 };
 
 gpsshogi::EvalValueStack * gpsshogi::
@@ -184,12 +190,9 @@ ProgressFeatureBase::save(const char *filename) const
 }
 
 void gpsshogi::
-ProgressFeatureBase::setWeight(const double *w)
+ProgressFeatureBase::setWeightScale(const double *w, double scale)
 {
-  if (! w[dimension()-1] > 0)
-    std::cerr << "error ProgressFeatureBase::setWeight max_progress is not positive.\n" << " " << w[dimension()-1] << "\n";
-  assert(w[dimension()-1] > 0);
-  scale = maxProgressConstraint()/w[dimension()-1];
+  this->scale = scale;
   int dim = 0;
   for (size_t i=0; i<all.size(); ++i) {
     all[i].setWeightScale(&w[0]+dim, scale);
@@ -198,6 +201,15 @@ ProgressFeatureBase::setWeight(const double *w)
   max_progress = round(w[dimension()-1]*scale);
 }
 
+void gpsshogi::
+ProgressFeatureBase::setWeight(const double *w)
+{
+  if (! w[dimension()-1] > 0)
+    std::cerr << "error ProgressFeatureBase::setWeight max_progress is not positive.\n" << " " << w[dimension()-1] << "\n";
+  assert(w[dimension()-1] > 0);
+  setWeightScale(w, maxProgressConstraint()/w[dimension()-1]);
+}
+
 void gpsshogi::
 ProgressFeatureBase::saveWeight(double *w) const
 {
@@ -215,9 +227,6 @@ ProgressFeatureBase::features(const NumEffectState& state, double& value,
 			   int offset) const
 {
   ome_eval_t eval(state);
-#ifndef L1BALL_NO_SORT
-  const size_t org_size = out.size();  
-#endif
   int dim = 0, progress = 0;
   for (size_t i=0; i<all.size(); ++i) {
     double his_value = 0.0;
@@ -255,9 +264,6 @@ ProgressFeatureBase::features(const NumEffectState& state, double& value,
 		  eval.openingValue(),
 		  eval.midgameValue(),
 		  eval.endgameValue());
-#ifndef L1BALL_NO_SORT
-  std::sort(out.begin()+org_size, out.end()); // todo: sort output of each feature
-#endif
 }
 
 void gpsshogi::
@@ -384,8 +390,8 @@ namespace gpsshogi
   public:
     enum { ONE_DIM = 2142 };
     ProgressPieceKingRelative() : EvalComponent(ONE_DIM * 2) { }
-    static int index(const Player player, const Position king,
-		     const Ptype ptype, const Position pos)
+    static int index(const Player player, const Square king,
+		     const Ptype ptype, const Square pos)
     {
       const int x = std::abs(pos.x() - king.x());
       const int y = (king.y() - pos.y()) *
@@ -393,23 +399,23 @@ namespace gpsshogi
       return (ptype - osl::PTYPE_PIECE_MIN) * 17 * 9 + (x * 17 + y);
     }
 
-    static int index(const Player player, const Position king,
+    static int index(const Player player, const Square king,
 		     const Piece piece)
     {
-      return index(player, king, piece.ptype(), piece.position());
+      return index(player, king, piece.ptype(), piece.square());
     }
     size_t maxActive() const { return 160; }
     const std::string name() const { return "ProgressPieceKingRelative"; };
     void featuresNonUniq(const NumEffectState &state,
 			 index_list_t &diffs, int offset) const
     {
-      const CArray<Position,2> kings = {{ 
-	  state.getKingPosition(BLACK),
-	  state.getKingPosition(WHITE),
+      const CArray<Square,2> kings = {{ 
+	  state.kingSquare(BLACK),
+	  state.kingSquare(WHITE),
 	}};
       for (int i = 0; i < osl::Piece::SIZE; ++i)
       {
-	const osl::Piece piece = state.getPieceOf(i);
+	const osl::Piece piece = state.pieceOf(i);
 	if (piece.ptype() == osl::KING || !piece.isOnBoard())
 	  continue;
 	Player pl = piece.owner();
@@ -451,8 +457,8 @@ namespace gpsshogi
 		  int &pawn, int &non_pawn) const
     {
       PieceMask mask = (state.effectedMask(PlayerTraits<Attacked>::opponent) &
-			state.getOnBoardMask(Attacked));
-      mask.reset(state.getKingPiece<Attacked>().number());
+			state.piecesOnBoard(Attacked));
+      mask.reset(state.kingPiece<Attacked>().number());
       mask_t black_ppawn = (state.promotedPieces().getMask<PAWN>() &
 			    mask.selectBit<PAWN>());
 
@@ -501,7 +507,7 @@ namespace gpsshogi
       PieceVector pieces;
       while (attacked.any())
       {
-	const Piece piece = state.getPieceOf(attacked.takeOneBit());
+	const Piece piece = state.pieceOf(attacked.takeOneBit());
 	pieces.push_back(piece);
       }
       for (size_t i=0; i+1<pieces.size(); ++i) {
@@ -515,15 +521,15 @@ namespace gpsshogi
     void featuresNonUniq(const NumEffectState &state,
 			 index_list_t &features, int offset) const
     {
-      PieceMask black_attacked = state.effectedMask(WHITE) & state.getOnBoardMask(BLACK);
-      black_attacked.reset(state.getKingPiece<BLACK>().number());
+      PieceMask black_attacked = state.effectedMask(WHITE) & state.piecesOnBoard(BLACK);
+      black_attacked.reset(state.kingPiece<BLACK>().number());
       mask_t black_ppawn = state.promotedPieces().getMask<PAWN>() & black_attacked.selectBit<PAWN>();
       black_attacked.clearBit<PAWN>();
       black_attacked.orMask(PtypeFuns<PAWN>::indexNum, black_ppawn);
       countOne(black_attacked, state, features, offset);
 
-      PieceMask white_attacked = state.effectedMask(BLACK) & state.getOnBoardMask(WHITE);
-      white_attacked.reset(state.getKingPiece<WHITE>().number());
+      PieceMask white_attacked = state.effectedMask(BLACK) & state.piecesOnBoard(WHITE);
+      white_attacked.reset(state.kingPiece<WHITE>().number());
       mask_t white_ppawn = state.promotedPieces().getMask<PAWN>() & white_attacked.selectBit<PAWN>();
       white_attacked.clearBit<PAWN>();
       white_attacked.orMask(PtypeFuns<PAWN>::indexNum, white_ppawn);
@@ -548,9 +554,9 @@ namespace gpsshogi
     static int index1(const NumEffectState &state, Piece piece)
     {
       const Ptype attack_ptype
-	= state.findCheapThreat(alt(piece.owner()), piece.position()).ptype();
-      const bool has_support = state.hasEffectBy(piece.owner(),
-						 piece.position());
+	= state.findCheapAttack(alt(piece.owner()), piece.square()).ptype();
+      const bool has_support = state.hasEffectAt(piece.owner(),
+						 piece.square());
       return (piece.ptype() + 
 	      (has_support ? 0 : PTYPE_SIZE)) * PTYPE_SIZE + attack_ptype;
     }
