@@ -11,9 +11,10 @@
 #include "osl/rating/ratingEnv.h"
 #include "osl/search/shouldPromoteCut.h"
 #include "osl/progress/effect5x3.h"
-#include "osl/move_generator/legalMoves.h"
 #include "osl/progress/ml/newProgress.h"
-#include "osl/apply_move/applyMove.h"
+#include "osl/move_generator/legalMoves.h"
+#include "osl/move_probability/featureSet.h"
+#include "osl/stl/hash_set.h"
 
 #include <boost/foreach.hpp>
 #include <algorithm>
@@ -21,11 +22,14 @@
 #include <sstream>
 #include <iostream>
 
+int gpsshogi::KisenAnalyzer::normal_depth = 2;
 int gpsshogi::KisenAnalyzer::quiesce_depth = 4;
 int gpsshogi::KisenAnalyzer::use_percent = 100;
 bool gpsshogi::KisenAnalyzer::compare_pass = false;
-bool gpsshogi::KisenAnalyzer::more_depth_in_opening = false;
-bool gpsshogi::PVGenerator::limit_sibling_10 = false;
+bool gpsshogi::KisenAnalyzer::bonanza_compatible_search = false;
+int gpsshogi::KisenAnalyzer::window_asymmetry = 1;
+int gpsshogi::PVGenerator::limit_sibling = 0;
+osl::vector<int> gpsshogi::KisenAnalyzer::priority_record;
 
 gpsshogi::KisenAnalyzer::
 RecordConfig::RecordConfig(size_t tid, size_t f, size_t l, const std::string& kisen, bool a)
@@ -37,6 +41,7 @@ gpsshogi::
 KisenAnalyzer::KisenAnalyzer(const RecordConfig& r, const OtherConfig& c, Result *out)
   : record(r), config(c), result(out)
 {
+  bonanza_compatible_search = (quiesce_depth < 0);
 }
 
 gpsshogi::
@@ -63,7 +68,7 @@ void gpsshogi::KisenAnalyzer::operator()()
   result->order_lb.clear();
   result->order_ub.clear();
   result->record_processed = result->skip_by_rating = 0;
-  result->node_count = 0;
+  result->node_count = result->siblings = 0;
   result->all_errors.clear();
 
   init();
@@ -72,13 +77,33 @@ void gpsshogi::KisenAnalyzer::operator()()
   if (config.min_rating)
     ipx.reset(new KisenIpxFile(KisenFile::ipxFileName(record.kisen_filename)));
 
+  assert(record.last >= record.first);
   vector<size_t> orders(record.last - record.first);
   for (size_t i=0; i<orders.size(); ++i)
     orders[i] = i + record.first;
   if (use_percent < 100) {
     std::random_shuffle(orders.begin(), orders.end());
-    orders.resize(orders.size()*use_percent/100);
+    const size_t limit = orders.size()*use_percent/100;
+    if (! priority_record.empty()) {
+      size_t p = 0, q = limit;
+      while (p < limit && q < orders.size()) {
+	if (std::binary_search(priority_record.begin(), priority_record.end(), 
+			       orders[p])) {
+	  ++p;
+	  continue;
+	}
+	if (! std::binary_search(priority_record.begin(), priority_record.end(),
+				 orders[q])) {
+	  ++q;
+	  continue;
+	}
+	std::swap(orders[p], orders[q]);
+	++p, ++q;
+      }
+    }
+    orders.resize(limit);
   }
+  hash_set<HashKey,osl::stl::hash<HashKey> > visited;
   BOOST_FOREACH(size_t i, orders) {
     if (record.allow_skip_in_cross_validation && isCrossValidation()) {
       if (i % 4)
@@ -104,15 +129,11 @@ void gpsshogi::KisenAnalyzer::operator()()
 
     NumEffectState state(kisen_file.getInitialState());
     const vector<Move> moves=kisen_file.getMoves(i);
-#ifdef LEARN_DEEP_SEARCH
-    Quiesce quiesce(config.my_eval, 2, 5);
-#else
-    Quiesce quiesce(config.my_eval, 1, quiesce_depth); 
-#endif
+    Quiesce quiesce(config.my_eval, normal_depth, quiesce_depth);
     osl::progress::ml::NewProgress progress(state);
     stat::Average record_errors;
     for (size_t j=0; j<moves.size(); j++) {
-      const Player turn = state.getTurn();
+      const Player turn = state.turn();
       // 自分の手番で相手の王が利きがある => 直前の手が非合法手
       if (state.inCheck(alt(turn))) {
 	std::cerr << "e"; // state;
@@ -122,15 +143,23 @@ void gpsshogi::KisenAnalyzer::operator()()
 	if (progress.progress16().value() > config.max_progress)
 	  break;
       }
-      if (! isCrossValidation() || (j+config.cross_validation_randomness) % 4 == 0) {
-	assert(j < moves.size());
-	forEachPosition(i, j, quiesce, state, progress.progress16(), moves[j]);
+      assert(j < moves.size());
+      HashKey key(state);
+      key[1] += moves[j].intValue();
+      const bool ignore = (j < 40) && ! visited.insert(key).second;
+      if (! ignore
+	  && (! (isCrossValidation() && record.allow_skip_in_cross_validation)
+	      || (j+config.position_randomness) % 4 == 0)) {
+	for (size_t k=j; k+1<moves.size(); ++k)
+	  quiesce.addBigram(moves[k], moves[k+1]);
+	forEachPosition(i, j, moves.size(), quiesce,
+			state, progress.progress16(), moves[j]);
 	if (isCrossValidation())
 	  record_errors.add(result->last_error);
       }
       result->node_count += quiesce.nodeCount();
       quiesce.clear();
-      ApplyMoveOfTurn::doMove(state, moves[j]);
+      state.makeMove(moves[j]);
       progress.update(state, moves[j]);
     }
 
@@ -311,11 +340,11 @@ Validator::isCrossValidation() const
 }
 
 void gpsshogi::
-Validator::forEachPosition(int /*record_id*/, int /*position_id*/, 
+Validator::forEachPosition(int /*record_id*/, int /*position_id*/, size_t,
 			   Quiesce& quiesce, const NumEffectState& state, 
 			   osl::Progress16, Move best_move)
 {
-  const int turn_coef = (state.getTurn() == BLACK) ? 1 : -1;
+  const int turn_sign = (state.turn() == BLACK) ? 1 : -1;
   double cur_errors = 0.0;
   result->last_error = 0.0;
   PVVector pv;
@@ -323,7 +352,7 @@ Validator::forEachPosition(int /*record_id*/, int /*position_id*/,
   int best_value;  
   {
     NumEffectState new_state = state;
-    ApplyMoveOfTurn::doMove(new_state, best_move);
+    new_state.makeMove(best_move);
     if (! quiesce.quiesce(new_state, best_value, pv))
       return;
   }
@@ -332,12 +361,14 @@ Validator::forEachPosition(int /*record_id*/, int /*position_id*/,
 
   MoveVector moves;
   LegalMoves::generate(state, moves);
-  if (compare_pass && ! state.inCheck())
-    moves.push_back(Move::PASS(state.getTurn()));
+  if (moves.size() < 2 || ! moves.isMember(best_move))
+    return;
+  if (! bonanza_compatible_search && compare_pass && ! state.inCheck())
+    moves.push_back(Move::PASS(state.turn()));
 
   size_t move_id = 0;
   vector<int> values; values.reserve(moves.size());
-  values.push_back(best_value*turn_coef);
+  values.push_back(best_value*turn_sign);
   for (MoveVector::const_iterator p=moves.begin(); p!=moves.end(); ++p, ++move_id) {
     if (*p == best_move)
       continue;
@@ -345,7 +376,7 @@ Validator::forEachPosition(int /*record_id*/, int /*position_id*/,
     int value;
     {
       NumEffectState new_state = state;
-      ApplyMoveOfTurn::doMove(new_state, *p);
+      new_state.makeMove(*p);
       pv.push_back(*p);
       const int width = (int)(config.window_by_pawn*config.my_eval->pawnValue());
       if (! quiesce.quiesce(new_state, value, pv, best_value - width, best_value + width))
@@ -353,13 +384,14 @@ Validator::forEachPosition(int /*record_id*/, int /*position_id*/,
     }
     if (abs(value) == quiesce.infty(BLACK))
       continue;
-    values.push_back(value*turn_coef);
-    cur_errors += SigmoidUtil::tx((value - best_value)*turn_coef, config.my_eval->pawnValue());
+    values.push_back(value*turn_sign);
+    cur_errors += SigmoidUtil::alphax((value - best_value)*turn_sign, config.sigmoid_alpha);
+    result->siblings += 1;
   }
   std::sort(values.begin(), values.end());
-  result->toprated.add(values.back()== best_value*turn_coef); // top?
-  result->toprated_strict.add(values.back() == best_value*turn_coef
-			      && values.size() > 1 && values[values.size()-2] != best_value*turn_coef);
+  result->toprated.add(values.back()== best_value*turn_sign); // top?
+  result->toprated_strict.add(values.back() == best_value*turn_sign
+			      && values.size() > 1 && values[values.size()-2] != best_value*turn_sign);
   result->order_lb.add(values.end() - std::lower_bound(values.begin(), values.end(), best_value));
   result->order_ub.add(values.end() - std::upper_bound(values.begin(), values.end(), best_value));
   result->werrors.add(cur_errors);
@@ -374,6 +406,9 @@ PVGenerator::PVGenerator(const std::string& pv_base,
   : KisenAnalyzer(r, c, out),
     pv_filename(pv_file(pv_base, r.thread_id))
 {
+  std::ostringstream ss;
+  ss << pv_base << r.thread_id << "-stat.txt";
+  stat_filename = ss.str();
 }
 
 gpsshogi::
@@ -395,124 +430,209 @@ PVGenerator::init()
   pw.reset(new PVFileWriter(pv_filename.c_str()));
 }
 
+gpsshogi::
+PVGenerator::SearchResult gpsshogi::
+PVGenerator::searchSiblings(Quiesce& quiesce, int width,
+			    const NumEffectState& state, 
+			    osl::Progress16 progress, Move best_move,
+			    int rand_seed, size_t accept_rank_threshold,
+			    int accept_piece_sacrifice_threshold,
+			    int& best_value, PVVector& best_pv, values_t& out,
+			    size_t& rank_search, size_t& rank_probability,
+			    int & outrange_better_move,
+			    int & outrange_other_move)
+{
+  const int ext2limit = 8;
+  out.clear();
+  outrange_better_move = outrange_other_move = 0;
+  // generate all
+  MoveVector moves;
+  rank_probability = rank_search = 0;
+  size_t rank_search_sacrifice = 0; 
+#ifdef USE_ALL_MOVES
+  LegalMoves::generate(state, moves);
+#else
+  static const move_probability::StandardFeatureSet& feature_set
+    = move_probability::StandardFeatureSet::instance();
+  MoveLogProbVector all;
+  MoveStack h;
+  move_probability::StateInfo info(state, progress, h);
+  feature_set.generateLogProb2(info, all);
+  size_t i=0;
+  for (; i<std::min(all.size(), (size_t)48); ++i) 
+    moves.push_back(all[i].move());
+  for (; i<all.size(); ++i)
+    if (((rand_seed + i) % 8 == 0) || bonanza_compatible_search)
+      moves.push_back(all[i].move());
+  const MoveLogProb *selected = all.find(best_move);
+  if (!selected)
+    return BestMoveNotFound;
+  if (moves.size() == 1)
+    return OneReply;
+  rank_probability = selected - &*all.begin();
+#endif
+  if (! bonanza_compatible_search && compare_pass && ! state.inCheck())
+    moves.push_back(Move::PASS(state.turn()));
+
+  // search bestmove
+  const int full_depth = quiesce.fullWidthDepth(), qdepth = quiesce.quiesceDepth();
+  best_pv.push_back(best_move);
+  {
+    if (! bonanza_compatible_search) {
+      if (rank_probability < 4 && full_depth > 0 && progress.value() < ext2limit)
+	quiesce.setDepth(full_depth+2, qdepth);
+      else if (rank_probability < 8 && full_depth > 0)
+	quiesce.setDepth(full_depth+1, qdepth);
+    }
+    NumEffectState new_state = state;
+    new_state.makeMove(best_move);
+    const bool good_pv = quiesce.quiesce(new_state, best_value, best_pv);
+    if (! good_pv)
+      return BestmoveSearchFailed;
+  }
+  if (abs(best_value) == quiesce.infty(BLACK))
+    return PVCheckmate;
+  const int turn_sign = (state.turn() == BLACK) ? 1 : -1;
+  const int alpha = best_value - width*turn_sign/window_asymmetry;
+  const int beta = best_value + width*turn_sign;
+  best_value *= turn_sign;
+  const int best_piece = best_pv.pieceValueOfTurn();
+  // search other moves
+  PVVector pv;
+  size_t move_id = 0;
+  out.reserve(moves.size());
+  NumEffectState copy;
+  for (MoveVector::const_iterator p=moves.begin(); p!=moves.end(); ++p, ++move_id) {
+    if (*p == best_move)
+      continue;
+    if (! p->isPass()
+	&& osl::search::ShouldPromoteCut::canIgnoreAndNotDrop(*p)
+	&& state.hasEffectAt(alt(p->player()), p->to()))
+      continue;
+    pv.clear();
+    int value;
+    {
+      if (! bonanza_compatible_search) {
+	if (move_id < 4 && rank_probability < 4 && full_depth > 0 && progress.value() < ext2limit)
+	  quiesce.setDepth(full_depth+2, qdepth);
+	else if (move_id < 8 && rank_probability < 8 && full_depth > 0)
+	  quiesce.setDepth(full_depth+1, qdepth);
+	else if (move_id >= 24 && move_id >= rank_probability && full_depth > 0)
+	  quiesce.setDepth(full_depth-1, qdepth);
+	else
+	  quiesce.setDepth(full_depth, qdepth);
+      }
+      copy.copyFrom(state);
+      copy.makeMove(*p);
+      pv.push_back(*p);
+      if (! quiesce.quiesce(copy, value, pv, alpha, beta))
+	continue;
+      value *= turn_sign;
+      if (! bonanza_compatible_search
+	  && quiesce.fullWidthDepth() < full_depth && value > best_value) {
+	pv.clear();
+	pv.push_back(*p);
+	quiesce.setDepth(full_depth, qdepth);
+	if (! quiesce.quiesce(copy, value, pv, alpha, beta))
+	  continue;
+      }
+    }
+    if (abs(value) == quiesce.infty(BLACK))
+      continue;
+    if (bonanza_compatible_search && abs(value - best_value) >= width){
+      if (value > best_value) 
+	++outrange_better_move;
+      else
+	++outrange_other_move;
+      continue;
+    }
+    if (value > best_value) {
+      ++rank_search;
+      if (pv.pieceValueOfTurn()
+	  >= best_piece + accept_piece_sacrifice_threshold) {
+	++rank_search_sacrifice;
+	if (! bonanza_compatible_search && rank_search_sacrifice >= accept_rank_threshold)
+	  return DifficultMove; 
+      }
+    }
+    out.push_back(std::make_pair(value, pv));
+  }
+  if (out.empty() && outrange_better_move + outrange_other_move == 0)
+    return OneReplyBySearch;
+  return SearchOK;
+}
+
 void gpsshogi::
-PVGenerator::forEachPosition(int record_id, int position_id, 
+PVGenerator::forEachPosition(int record_id, int position_id, size_t position_size,
 			     Quiesce& quiesce, const NumEffectState& state, 
 			     osl::Progress16 progress, Move best_move)
 {
-  const bool in_opening = more_depth_in_opening && progress.value() == 0
-    && ((record_id+position_id)%3 == 0);
-
-  PVVector pv;
-  pv.push_back(best_move);
-  int best_value;
-  const int full_depth = quiesce.fullWidthDepth();
-  const int quiesce_depth = quiesce.quiesceDepth();
-  {
-    NumEffectState new_state = state;
-    ApplyMoveOfTurn::doMove(new_state, best_move);
-    if (in_opening)
-      quiesce.setDepth(0, quiesce_depth);
-    const bool good_pv = quiesce.quiesce(new_state, best_value, pv);
-    if (in_opening)
-      quiesce.setDepth(full_depth, quiesce_depth);
-    if (! good_pv)
-      return;
-#ifdef DEBUG_ALL
-    std::cerr << "*BEST*\n" << new_state << pv;
-#endif
-  }
-  if (abs(best_value) == quiesce.infty(BLACK))
-    return;
-#ifdef WATCH_PV
-  std::cerr << state << pv;
-#endif
-  pw->newPosition(record_id, position_id);
-  pw->addPv(pv);
-
-  MoveVector moves;
-  LegalMoves::generate(state, moves);
-  if (compare_pass && ! state.inCheck())
-    moves.push_back(Move::PASS(state.getTurn()));
-
-  const int turn_coef = (state.getTurn() == BLACK) ? 1 : -1;
-  double cur_errors = 0.0;
-  size_t move_id = 0;
-  typedef vector<std::pair<int,PVVector> > vector_t;
-  vector_t values;
-  values.reserve(in_opening ? moves.size()*2 : moves.size());
-  for (int d=0; d<(in_opening ? 2 : 1); ++d) 
-  {
-    if (in_opening)
-      quiesce.setDepth((d == 0) ? 2 : 0, quiesce_depth);
-    for (MoveVector::const_iterator p=moves.begin(); p!=moves.end(); ++p, ++move_id) {
-      if (*p == best_move)
-	continue;
-      if (! p->isPass()
-	  && osl::search::ShouldPromoteCut::canIgnoreAndNotDrop(*p)
-	  && state.hasEffectBy(alt(p->player()), p->to()))
-	continue;
-
-      pv.clear();
-      int value;
-      {
-	NumEffectState new_state = state;
-	ApplyMoveOfTurn::doMove(new_state, *p);
-	pv.push_back(*p);
-	const int width = (int)(config.window_by_pawn*config.my_eval->pawnValue());
-	if (! quiesce.quiesce(new_state, value, pv, best_value - width, best_value + width))
-	  continue;
-      }
-      if (abs(value) == quiesce.infty(BLACK))
-	continue;
-      if (d > 0 && pv.size() > 2)
-	break;
-#ifdef WATCH_PV
-      if (p - moves.begin() < 4)
-	std::cerr << pv;
-#endif
-#ifdef DEBUG_PV
-      {
-	HistoryState stack;
-	Analyzer::makeLeaf(stack, pv);
-	if (stack.state.inCheck(BLACK) || stack.state.inCheck(WHITE)) {
-	  // continue;
-	  std::cerr << state << pv << value << "\n" << record_id << " "
-		    << position_id << " " << move_id << "\n";
-	}
-
-	assert(! stack.state.inCheck(BLACK));
-	assert(! stack.state.inCheck(WHITE));
-      }
-#endif
-      cur_errors += SigmoidUtil::tx((value - best_value)*turn_coef, config.my_eval->pawnValue());
-      values.push_back(std::make_pair(value*turn_coef, pv));
+  const int width = config.window_by_pawn*config.my_eval->pawnValue();
+  const int full_depth = quiesce.fullWidthDepth(), qdepth = quiesce.quiesceDepth();
+  int best_value, outrange_better_move, outrange_other_move;
+  PVVector best_pv;
+  values_t values;
+  size_t rank_search, rank_probability;
+  SearchResult analyses_result = searchSiblings
+    (quiesce, width, state, progress, best_move,
+     record_id+position_id+config.position_randomness+record.thread_id,
+     (position_size - position_id < 10) ? 1 : 16,
+     256,
+     best_value, best_pv, values, rank_search, rank_probability,
+     outrange_better_move, outrange_other_move);
+  quiesce.setDepth(full_depth, qdepth);
+  if (analyses_result != SearchOK) {
+    if (analyses_result == DifficultMove) {
+      result->toprated.add(false);
+      result->toprated_strict.add(false);
+    } else if (analyses_result == OneReplyBySearch) {
+      result->werrors.add(0);
+      result->toprated.add(true);
+      result->toprated_strict.add(true);
     }
-  }
-  if (in_opening)	// restore
-    quiesce.setDepth(full_depth, quiesce_depth);
-  if (values.empty())
     return;
-  if (! limit_sibling_10) 
+  }
+
+  double cur_errors = 0.0;
+  BOOST_FOREACH(const value_and_pv_t& p, values) {
+    cur_errors += SigmoidUtil::alphax(p.first - best_value,
+				      config.sigmoid_alpha);
+  }
+  cur_errors += outrange_better_move;
+  result->siblings += values.size();
+  result->siblings += outrange_better_move + outrange_other_move;
+  if (values.empty()) {
+    result->werrors.add(cur_errors);
+    result->toprated.add(outrange_better_move == 0);
+    result->toprated_strict.add(outrange_better_move == 0);
+    return;
+  }
+  pw->newPosition(record_id, position_id);
+  pw->addPv(best_pv);
+  
+  if (limit_sibling <= 0) 
   {
-    vector_t::const_iterator p
+    values_t::const_iterator p
       = std::max_element(values.begin(), values.end());
-    result->toprated.add(p->first <= best_value*turn_coef);
-    result->toprated_strict.add(p->first < best_value*turn_coef);    
+    result->toprated.add(outrange_better_move == 0 && p->first <= best_value);
+    result->toprated_strict.add(outrange_better_move == 0 && p->first < best_value);
   }
   else
   {
-    std::sort(values.begin(), values.end());
-    result->toprated.add(values.back().first <= best_value*turn_coef);
-    result->toprated_strict.add(values.back().first < best_value*turn_coef);
+    std::sort(values.begin(), values.end()); // ascending order
+    result->toprated.add(outrange_better_move ==0 && values.back().first <= best_value);
+    result->toprated_strict.add(outrange_better_move == 0 && values.back().first < best_value);
     size_t i=0;
     for (;i<values.size(); ++i) 
-      if (values[i].first > best_value*turn_coef)
+      if (values[i].first > best_value)
 	break;
-    values.resize(std::min(i+10, values.size()));
+    values.resize(std::min(i+limit_sibling, values.size()));
+  }
+  for (size_t i=0; i<values.size(); ++i) {
+    bool good_move = values[i].first > best_value - config.window_by_pawn*config.my_eval->pawnValue();
+    if (good_move || ((record_id + position_id + i + config.position_randomness) % 8 == 0))
+      pw->addPv(values[i].second);
   }
-  for (size_t i=0; i<values.size(); ++i)
-    pw->addPv(values[i].second);
   result->werrors.add(cur_errors);
   result->last_error = cur_errors;
 }
