@@ -1,13 +1,14 @@
 #include "board.h"
 #include "osl/state/numEffectState.h"
-#include "osl/apply_move/applyMove.h"
 #include "osl/eval/progressEval.h"
 #include "osl/eval/ml/openMidEndingEval.h"
 #include "osl/progress/effect5x3.h"
 #include "osl/record/csa.h"
 #include "osl/record/usi.h"
 #include "osl/threatmate/richPredictor.h"
+#include "osl/misc/iconvConvert.h"
 #include <iostream>
+using osl::IconvConvert;
 
 gpsshell::
 Board::Board()
@@ -17,6 +18,12 @@ Board::Board()
     white_color(osl::record::Color::NONE),
     last_color(osl::record::Color::NONE)
 {
+  static const char *env_term = getenv("TERM");
+  if (env_term && std::string(env_term) != "dumb") {
+    // setBlackColor(osl::record::Color::Green);
+    setWhiteColor(osl::record::Color::Blue);
+    setLastMoveColor(osl::record::Color::Red);
+  }
 }
 
 gpsshell::
@@ -90,11 +97,10 @@ osl::NumEffectState gpsshell::
 Board::getState() const
 {
   osl::NumEffectState state = initial;
-  // TODO Ask Kaneko osl::NumEffectState state(osl::state::SimpleState(HIRATE)); // no function error
   for (size_t i=0; i < current_move; i++)
   {
      osl::Move move = moves[i];
-     osl::ApplyMoveOfTurn::doMove(state, move);
+     state.makeMove(move);
   }
   return state;
 }
@@ -146,7 +152,7 @@ Board::showState() const
       std::cout << std::endl;
     }
     if (id < comments.size())
-      std::cout << comments[id] << std::endl;
+      std::cout << IconvConvert::eucToLang(comments[id]) << std::endl;
   }
 }
 
@@ -158,16 +164,18 @@ Board::showEval(const std::string& name) const
   {
     osl::progress::ml::NewProgress progress(state);
     osl::eval::ml::OpenMidEndingEval eval(state);
-    std::cout << "ɾ: "
+    std::cout << IconvConvert::eucToLang("ɾ: ")
 	      << eval.value() << " ("
 	      << eval.openingValue()*progress.maxProgress() << " "
 	      << eval.midgameValue()*progress.maxProgress() << " "
+	      << eval.midgame2Value()*progress.maxProgress() << " "
 	      << eval.endgameValue()*progress.maxProgress() << ")"
+	      << " (" << eval.progressIndependentValue() << ")"
 	      << std::endl;
-    std::cout << "ʹ: "
+    std::cout << IconvConvert::eucToLang("ʹ: ")
 	      << progress.progress() << " / "
 	      << progress.maxProgress() << " "
-	      << progress.progress16().value() << " "
+	      << progress.progress16().value() << " / 16, "
 	      << progress.progress16(osl::BLACK).value() << " "
 	      << progress.progress16(osl::WHITE).value() << " "
 	      << progress.progressAttack(osl::BLACK).value() << " "
@@ -187,7 +195,7 @@ Board::showEval(const std::string& name) const
     if (name != "progress")
       std::cerr << name << " is not supported. use progress instead\n";    
     osl::eval::ProgressEval eval(state);
-    std::cout << "ɾ: "
+    std::cout << IconvConvert::eucToLang("ɾ: ")
 	      << eval.value() << " ("
 	      << eval.openingValue()*16 << " "
 	      << eval.endgameValue()*16 << " "
@@ -195,7 +203,7 @@ Board::showEval(const std::string& name) const
 	      << eval.minorPieceValue() << ")"
 	      << std::endl;
     osl::progress::Effect5x3 progress(state);
-    std::cout << "ʹ: "
+    std::cout << IconvConvert::eucToLang("ʹ: ")
 	      << progress.progress16().value() << " "
 	      << progress.progress16(osl::BLACK).value() << " "
 	      << progress.progress16(osl::WHITE).value() << " "
@@ -203,7 +211,7 @@ Board::showEval(const std::string& name) const
   }
   if (current_move > 0 && current_move-1 < moves.size()) {
     osl::threatmate::RichPredictor predictor;
-    std::cout << "ͤΨ: "
+    std::cout << IconvConvert::eucToLang("ͤΨ: ")
 	      << predictor.predict(state, moves[current_move-1])
 	      << std::endl;
   }
