@@ -11,13 +11,11 @@
 #include "osl/search/searchState2.h"
 #include "osl/search/searchMonitor.h"
 #include "osl/search/analyzer/recordSet_.h"
-#include "osl/eval/progressEval.h"
-#include "osl/apply_move/applyMove.h"
+#include "osl/eval/ml/openMidEndingEval.h"
 #include "osl/game_playing/alphaBetaPlayer.h"
 #include "osl/move_generator/legalMoves.h"
 #include "osl/container/moveVector.h"
 #include "osl/effect_util/effectUtil.h"
-#include "osl/apply_move/applyMove.h"
 #include "osl/sennichite.h"
 #include "osl/record/csa.h"
 #include "osl/record/ki2.h"
@@ -25,27 +23,18 @@
 #include "gpsshogi/gui/util.h"
 #include "gpsshogi/gui/board.h"
 
+#include <QtGui>
 #include <qlayout.h>
 #include <qlistview.h>
 #include <qspinbox.h>
 #include <qcombobox.h>
-#if QT_VERSION >= 0x040000
-#include <Q3PopupMenu>
-#include <Q3Action>
-#include <Q3ProgressDialog>
-#else
-#include <qpopupmenu.h>
-#include <qaction.h>
-#include <qprogressdialog.h>
-#define Q3PopupMenu QPopupMenu
-#define Q3Action QAction
-#define Q3ProgressDialog QProgressDialog
-#endif
+#include <QProgressDialog>
 #include <qpushbutton.h>
 #include <qmessagebox.h>
 #include <qthread.h>
 #include <qapplication.h>
 #include <qtextcodec.h>
+#include <QTableView>
 
 #include <boost/scoped_ptr.hpp>
 #include <sstream>
@@ -67,15 +56,15 @@ static void generateMoves(const osl::SimpleHashRecord *record,
   osl::search::MoveGenerator generator;
   moves.clear();
 
-  osl::eval::ProgressEval eval(state);
+  osl::eval::ml::OpenMidEndingEval eval(state);
   generator.init(2000, record, eval, state, true, osl::Move());
 
   static osl::search::SearchState2::checkmate_t c;
   osl::search::SearchState2 search_state(state, c);
   search_state.setHistory(history);
-  generator.generateAll(state.getTurn(), search_state, moves);
+  generator.generateAll(state.turn(), search_state, moves);
   if (! record->inCheck())
-    moves.push_back(osl::MoveLogProb(osl::Move::PASS(state.getTurn()), 200));
+    moves.push_back(osl::MoveLogProb(osl::Move::PASS(state.turn()), 200));
 }
 static bool hasChildAfterMove(const osl::SimpleState& state,
 			      const osl::search::SimpleHashTable& table,
@@ -84,57 +73,18 @@ static bool hasChildAfterMove(const osl::SimpleState& state,
 			      const osl::SimpleHashRecord *child_record,
 			      const osl::Move move)
 {
-  osl::SimpleState child_state = state;
-  osl::ApplyMoveOfTurn::doMove(child_state, move);
+  osl::NumEffectState child_state(state);
+  child_state.makeMove(move);
   osl::MoveStack new_history = history;
   new_history.push(move);
   osl::MoveLogProbVector moves;
   generateMoves(child_record, child_state, new_history, moves);
   for (size_t i=0; i<moves.size(); ++i)
-    if (table.find(child_key.newHashWithMove(moves[i].getMove())))
+    if (table.find(child_key.newHashWithMove(moves[i].move())))
       return true;
   return false;
 }
 
-
-
-class MoveListItem : public Q3ListViewItem
-{
-public:
-  MoveListItem(Q3ListView *parent, osl::Move m, int v, QString best_moves)
-    : Q3ListViewItem(parent, gpsshogi::gui::Util::moveToString(m),
-		    QString("%1").arg(v), best_moves),
-      move(m), value(v) {
-  }
-  int compare(Q3ListViewItem *i, int col, bool ascending) const;
-  osl::Move getMove() const {
-    return move;
-  }
-private:
-  osl::Move move;
-  int value;
-};
-
-int MoveListItem::compare(Q3ListViewItem *i, int col, bool ascending) const
-{
-  if (col == 0)
-  {
-    osl::Move m2 = ((MoveListItem *)i)->getMove();
-    return gpsshogi::gui::Util::compare(getMove(), m2);
-  }
-  else if (col == 1)
-  {
-    MoveListItem *mi = (MoveListItem *)i;
-    if (value == mi->value)
-      return 0;
-    else if (value < mi->value)
-      return -1;
-    else
-      return 1;
-  }
-  else
-    return key(col, ascending).compare(i->key(col, ascending));
-}
 
 
 class QuiescenceDialog : public QDialog
@@ -264,9 +214,9 @@ void SearchMoveTreeItem::setOpen(bool open)
 
     osl::MoveVector history;
     getMovesToCurrent(history);
-    osl::SimpleState state = tree->initialState();
+    osl::NumEffectState state(tree->initialState());
     for (size_t i=0; i<history.size(); ++i)
-      osl::ApplyMoveOfTurn::doMove(state, history[i]);
+      state.makeMove(history[i]);
   
     osl::MoveLogProbVector moves;
     osl::MoveStack move_history;
@@ -279,30 +229,30 @@ void SearchMoveTreeItem::setOpen(bool open)
 
     for (size_t i=0; i<moves.size(); ++i)
     {
-      const osl::Move move = moves[i].getMove();
+      const osl::Move move = moves[i].move();
       const osl::HashKey child_key = key.newHashWithMove(move);
       const osl::SimpleHashRecord *child_record = tree->getTable().find(child_key);
       if (record->threatmate().maybeThreatmate(move.player()) &&
 	  child_record &&
 	  child_record->lowerLimit() == osl::search::SearchTable::CheckmateSpecialDepth &&
-	  child_record->bestMove().getMove().isNormal())
+	  child_record->bestMove().isNormal())
 	continue;
 
       int winState = NO_CHECKMATE;
       if (record->lowerLimit() == osl::search::SearchTable::CheckmateSpecialDepth &&
-	  record->bestMove().getMove().isNormal() &&
-	  move == record->bestMove().getMove())
+	  record->bestMove().isNormal() &&
+	  move == record->bestMove().move())
 	winState = CHECKMATE_WIN;
       else if (child_record &&
 	       child_record->lowerLimit() == osl::search::SearchTable::CheckmateSpecialDepth &&
-	       child_record->bestMove().getMove().isNormal())
+	       child_record->bestMove().isNormal())
 	winState = CHECKMATE_LOSE;
 
       SearchMoveTreeItem *item =
 	new SearchMoveTreeItem(this,
 			       move,
-			       moves[i].getLogProb(),
-			       move == record->bestMove().getMove(),
+			       moves[i].logProb(),
+			       move == record->bestMove().move(),
 			       winState,
 			       child_record);
       if (child_record)
@@ -423,29 +373,29 @@ void SearchMoveTree::addRootItem(const osl::search::SimpleHashRecord *root_recor
   const osl::HashKey key(initial_state);
   for (size_t i=0; i<moves.size(); ++i)
   {
-    const osl::HashKey child_key = key.newHashWithMove(moves[i].getMove());
+    const osl::HashKey child_key = key.newHashWithMove(moves[i].move());
     const osl::SimpleHashRecord *record = table->find(child_key);
     if (record == 0)
       continue;
     int winState = SearchMoveTreeItem::NO_CHECKMATE;
     if (root_record->lowerLimit() == osl::search::SearchTable::CheckmateSpecialDepth &&
-	root_record->bestMove().getMove().isNormal() &&
-	moves[i].getMove() == root_record->bestMove().getMove())
+	root_record->bestMove().isNormal() &&
+	moves[i].move() == root_record->bestMove().move())
       winState = SearchMoveTreeItem::CHECKMATE_WIN;
     else if (record &&
 	     record->lowerLimit() == osl::search::SearchTable::CheckmateSpecialDepth &&
-	     record->bestMove().getMove().isPass())
+	     record->bestMove().isPass())
       winState = SearchMoveTreeItem::CHECKMATE_LOSE;
 
     Q3ListViewItem *item =
       new SearchMoveTreeItem(this,
-			     moves[i].getMove(),
-			     moves[i].getLogProb(),
-			     moves[i].getMove() == root_record->bestMove().getMove(),
+			     moves[i].move(),
+			     moves[i].logProb(),
+			     moves[i].move() == root_record->bestMove().move(),
 			     winState,
 			     record);
     if (record && hasChildAfterMove(initial_state, *table, history, 
-				    child_key, record, moves[i].getMove()))
+				    child_key, record, moves[i].move()))
       item->setExpandable(true);
   }
 }
@@ -466,7 +416,7 @@ void SearchMoveTree::showQuiescenceRecord()
   {
     if (moves[i].isInvalid())
       break;
-    osl::ApplyMoveOfTurn::doMove(newState, moves[i]);
+    newState.makeMove(moves[i]);
   }
   QuiescenceDialog *dialog = new QuiescenceDialog(osl::HashKey(newState),
 						  newState,
@@ -525,7 +475,7 @@ void SearchMoveTree::showMovesInKanji()
       : osl::record::ki2::show(moves[i], state);
     moves_text.append(codec->toUnicode(euc_move.c_str(), euc_move.length()))
       .append("\n");
-    osl::ApplyMoveOfTurn::doMove(state, moves[i]);
+    state.makeMove(moves[i]);
   }
   
   QMessageBox::information(this, "Moves",
@@ -570,37 +520,34 @@ void SearchMoveTree::showSubtree()
   dialog->exec();
 }
 
-void SearchMoveTree::showContextMenu(Q3PopupMenu * contextMenu)
+void SearchMoveTree::buildContextMenu(QMenu *menu)
 {
-  contextMenu->insertSeparator();
-  Q3Action *showQRecordAction = new Q3Action("Show Quiescence Record", 0,
-					     contextMenu);
-  showQRecordAction->addTo(contextMenu);
+  MoveTree::buildContextMenu(menu);
+  menu->insertSeparator();
+  QAction *showQRecordAction = new QAction(tr("Show Quiescence Record"), this);
+  menu->addAction(showQRecordAction);
   connect(showQRecordAction, SIGNAL(activated()),
 	  this, SLOT(showQuiescenceRecord()));
 
-  Q3Action *showTreeAction = new Q3Action("Show This Subtree", 0,
-					  contextMenu);
-  showTreeAction->addTo(contextMenu);
+  QAction *showTreeAction = new QAction(tr("Show This Subtree"), this);
+  menu->addAction(showTreeAction);
   connect(showTreeAction, SIGNAL(activated()),
 	  this, SLOT(showSubtree()));
 
-  contextMenu->insertSeparator();
-  Q3Action *showRecordAction = new Q3Action("Dump Record Data", 0,
-					    contextMenu);
-  showRecordAction->addTo(contextMenu);
+  menu->insertSeparator();
+  QAction *showRecordAction = new QAction(tr("Dump Record Data"), this);
+  menu->addAction(showRecordAction);
   connect(showRecordAction, SIGNAL(activated()),
 	  this, SLOT(showRecord()));
 
-  Q3Action *showMovesAction = new Q3Action("Show moves to this node", 0,
-					    contextMenu);
-  showMovesAction->addTo(contextMenu);
+  QAction *showMovesAction = new QAction(tr("Show moves to this node"), this);
+  menu->addAction(showMovesAction);
   connect(showMovesAction, SIGNAL(activated()),
 	  this, SLOT(showMoves()));
 
-  Q3Action *showMovesInKanjiAction = new Q3Action("Show moves to this node in Kanji", 0,
-					    contextMenu);
-  showMovesInKanjiAction->addTo(contextMenu);
+  QAction *showMovesInKanjiAction =
+    new QAction(tr("Show moves to this node in Kanji"), this);
+  menu->addAction(showMovesInKanjiAction);
   connect(showMovesInKanjiAction, SIGNAL(activated()),
 	  this, SLOT(showMovesInKanji()));
 }
@@ -639,22 +586,22 @@ AnalysisViewer::AnalysisViewer(QWidget *parent, const char *name)
 {
   moveTree = new SearchMoveTree(this);
 
-  moveList = new Q3ListView(this);
-  moveList->addColumn("Move");
-  moveList->addColumn("Value");
-  moveList->addColumn("Principal Variation");
-  moveList->setColumnAlignment(1, Qt::AlignRight);
-
+  pvView = new QTableView(this);
+  pvView->verticalHeader()->hide();
+  pvView->setShowGrid(false);
+  pvView->setSortingEnabled(true);
+  sortedPvModel.reset(new PvProxyModel);
+  
   QVBoxLayout *moveLayout = new QVBoxLayout;
   moveLayout->addWidget(moveTree, 2);
-  moveLayout->addWidget(moveList, 1);
+  moveLayout->addWidget(pvView, 1);
 
   QHBoxLayout *mainLayout = new QHBoxLayout(this);
   mainLayout->addWidget(board);
   mainLayout->addLayout(moveLayout);
 
-  connect(moveList, SIGNAL(doubleClicked(Q3ListViewItem *, const QPoint&, int)),
-	  this, SLOT(openMoves(Q3ListViewItem *)));
+  connect(pvView, SIGNAL(doubleClicked(const QModelIndex &)),
+          this, SLOT(openMoves(const QModelIndex &)));
   connect(board, SIGNAL(statusChanged()),
 	  this, SLOT(notifyState()));
 
@@ -716,7 +663,7 @@ public:
     if (first == last || cur != *first)
       pv.push_back(cur);
     pv.insert(pv.end(), first, last);
-    pv_value = value * osl::playerToMul(root.getTurn());
+    pv_value = value * osl::playerToMul(root.turn());
     this->node_count = node_count;
     this->elapsed = elapsed;
     update();
@@ -755,16 +702,15 @@ public:
 bool AnalysisViewer::analyzeWithSavedValue(const osl::state::SimpleState& s,
 					   const osl::stl::vector<osl::Move> &moves)
 {
-  moveList->clear();
   moveTree->clear();
-  initialState = s;
+  initialState.copyFrom(s);
   osl::game_playing::GameState gameState(s);  
   for (int i = 0; i < (int)moves.size(); i++)
   {
     if (moves[i].isInvalid())
       break;
     gameState.pushMove(moves[i]);
-    osl::ApplyMoveOfTurn::doMove(initialState, moves[i]);
+    initialState.makeMove(moves[i]);
   }
   board->setState(initialState);
 
@@ -802,9 +748,10 @@ bool AnalysisViewer::analyzeWithSavedValue(const osl::state::SimpleState& s,
 #ifdef OSL_NOTHREADS
   player->selectBestMove(gameState, 0, 0, time);
 #else
-  Q3ProgressDialog progress("Searching...........................................", "Abort Search", time*8,
-			    this, 0, true);
-  QLabel *search_info = new QLabel(QString::fromUtf8("Searching"));
+  QProgressDialog progress(tr("Searching..........................................."),
+                           tr("Abort Search"), 0, time*8,
+                           this);
+  QLabel *search_info = new QLabel(tr("Searching"));
   progress.setLabel(search_info);
   
   boost::shared_ptr<osl::search::SearchMonitor> monitor
@@ -818,14 +765,14 @@ bool AnalysisViewer::analyzeWithSavedValue(const osl::state::SimpleState& s,
     timespec rq = { 0, 125000000 }, rm;
     while (nanosleep(&rq, &rm) == -1) 
       rq = rm;
-    progress.setProgress(i);
+    progress.setValue(i);
     qApp->processEvents();
 
     if (progress.wasCanceled())
     {
       player->stopSearchNow();
       thread.wait();
-      progress.setProgress(time);
+      progress.setValue(time);
       break;
     }
 #if QT_VERSION >= 0x040000
@@ -838,7 +785,7 @@ bool AnalysisViewer::analyzeWithSavedValue(const osl::state::SimpleState& s,
   }
 
   thread.wait();
-  progress.setProgress(time*4);
+  progress.setValue(time*4);
 #endif
   const osl::search::SimpleHashTable& table = *player->table();
   const osl::HashKey key(gameState.state());
@@ -872,29 +819,11 @@ void AnalysisViewer::saveSearchValue(const SearchDialogImpl& dialog)
 void AnalysisViewer::bestMoves(const osl::NumEffectState& state, 
 			       const osl::search::SimpleHashTable& table)
 {
-  {
-    const osl::HashKey key(state);
-    osl::MoveVector moves;
-    osl::LegalMoves::generate(state, moves);
-    for (osl::MoveVector::const_iterator p=moves.begin();
-	 p!=moves.end(); ++p) {
-      if (table.find(key.newHashWithMove(*p))) {
-	osl::MoveVector pv;
-	const osl::HashKey new_hash = key.newHashWithMove(*p);
-	table.getPV(new_hash, pv);
-	QString best_moves;
-	for (size_t i=0; i<pv.size(); ++i) {
-	  if (best_moves != "")
-	    best_moves.append(", ");
-	  best_moves.append(gpsshogi::gui::Util::moveToString(pv[i]));
-	}
-	const int value = getRecordValue(table.find(new_hash), 
-					 p->player());
-	new MoveListItem(moveList, *p, value, best_moves);
-      }
-    }
-    return;
-  }
+  pvModel.reset(new PvModel(state, table, this));
+  sortedPvModel->setSourceModel(pvModel.get());
+  pvView->setModel(sortedPvModel.get());
+  pvView->resizeColumnsToContents();
+  pvView->sortByColumn(0, Qt::AscendingOrder);
 }
 
 int AnalysisViewer::getRecordValue(const osl::search::SimpleHashRecord *r,
@@ -919,13 +848,10 @@ const osl::search::SimpleHashRecord *AnalysisViewer::selectedRecord()
   return 0;
 }
 
-void AnalysisViewer::openMoves(Q3ListViewItem *i)
+void AnalysisViewer::openMoves(const QModelIndex &index)
 {
-  SearchMoveTreeItem *item = (SearchMoveTreeItem *)i;
-  if (!item)
-    return;
-
-  osl::Move move = item->getMove();
+  QVariant v = pvModel->data(index, PvModel::FirstMoveRole);
+  osl::Move move = osl::Move::makeDirect(v.toInt());
 
   for (SearchMoveTreeItem *child = (SearchMoveTreeItem *)moveTree->firstChild();
        child; child = (SearchMoveTreeItem *)child->nextSibling())
