@@ -20,7 +20,7 @@ public:
   int moveCount() const {
     return numMoves;
   }
-  osl::state::SimpleState getStateAndMovesToCurrent(osl::stl::vector<osl::Move> &moves);
+  osl::NumEffectState getStateAndMovesToCurrent(osl::stl::vector<osl::Move> &moves);
   void toLastState();
 private slots:
 #if QT_VERSION >= 0x040000
