@@ -17,7 +17,7 @@ public:
   BoardTabChild(QWidget *parent = 0, const char *name = 0);
   void toInitialState();
   osl::Player turn() const;
-  const osl::state::SimpleState& getState();
+  const osl::NumEffectState& getState();
   void toggleOrientation();
   void setOrientation(bool sente);
   bool isSenteView() const;
@@ -28,7 +28,7 @@ public:
   void enableEffect(bool on);
   bool effectEnabled();
   void highlightLastMove(bool on);
-  virtual osl::state::SimpleState getStateAndMovesToCurrent(osl::stl::vector<osl::Move> &moves) = 0;
+  virtual osl::NumEffectState getStateAndMovesToCurrent(osl::stl::vector<osl::Move> &moves) = 0;
   virtual int getLimit() { return 1000; };
 public slots:
   QWidget *moveGenerateDialog();
@@ -40,7 +40,7 @@ signals:
 protected:
   virtual osl::Move getNextMove() { return osl::Move::INVALID(); }
   gpsshogi::gui::Board *board;
-  osl::state::SimpleState initialState;
+  osl::NumEffectState initialState;
 protected slots:
   void notifyState();
 };
