@@ -12,13 +12,14 @@
 #include "osl/record/csaIOError.h"
 #include "osl/record/kisen.h"
 #include "osl/record/ki2.h"
-#include "osl/apply_move/applyMove.h"
 
 #include "gpsshogi/gui/util.h"
 
 #include <qlayout.h>
+#include <QListWidget>
+#include <QTableWidget>
+#include <QThread>
 #if QT_VERSION >= 0x040000
-#include <Q3ListBox>
 #include <Q3ListView>
 #include <QDateTime>
 #else
@@ -35,15 +36,13 @@
 #include <qmessagebox.h>
 #include <qlabel.h>
 #include <qpushbutton.h>
-#include <qlayout.h>
-#include <qtextcodec.h>
 
 volatile bool KifuViewer::analyzeOnlineEnabled = false;
 volatile bool KifuViewer::stateForAnalysisChanged = false;
 boost::scoped_ptr<osl::NumEffectState> KifuViewer::stateForAnalysis;
 boost::scoped_ptr<KifuViewer::AnalysisOnlineThread> KifuViewer::thread;
 KifuViewer::AnalysisOnlineDialog *KifuViewer::analysisOnlineDialog = 0;
-static boost::mutex analysisOnlineMutex;
+static QMutex analysisOnlineMutex;
 
 
 KifuViewer::KifuViewer(QWidget *parent, const char *name)
@@ -52,13 +51,9 @@ KifuViewer::KifuViewer(QWidget *parent, const char *name)
     watchFileEnabled(false)
 {
   list = new MoveList(this);
-  players = new Q3ListBox(this);
-  timeView = new Q3ListView(this);
-  timeView->addColumn("");
-  timeView->addColumn(QString::fromUtf8("消費時間"));
-  timeView->setColumnAlignment(1, Qt::AlignRight);
-  timeView->setSorting(-1);
-  moveInfoView = new Q3ListBox(this);
+  players = new QListWidget(this);
+  timeView = new QTableWidget(2, 1, this);
+  moveInfoView = new QListWidget(this);
   commentView = new QTextBrowser(this);
   updateTime();
 
@@ -74,7 +69,7 @@ KifuViewer::KifuViewer(QWidget *parent, const char *name)
 
   timer = new QTimer(this);
 
-  connect(list, SIGNAL(selected(int)),
+  connect(list, SIGNAL(currentRowChanged(int)),
 	  this, SLOT(updateIndex(int)));
   connect(board, SIGNAL(moved(osl::Move)),
 	  this, SLOT(boardMoved(osl::Move)));
@@ -118,11 +113,10 @@ void KifuViewer::forward()
 
   if (!move.isPass())
   {
-    list->setSelected(index, true);
-    list->ensureCurrentVisible();
     ignoreMoveSignal = true;
     board->move(move);
     ignoreMoveSignal = false;
+    list->setCurrentItem(list->item(index));
   }
   manualIndex = -1;
   emit indexUpdate(index);
@@ -158,27 +152,26 @@ void KifuViewer::updateStateToIndex(int n)
 {
   if (n < (int)list->numMoves())
   {
-    osl::state::SimpleState state(initialState);
+    osl::NumEffectState state(initialState);
     for (int i = 0; i <= n; i++)
     {
-      osl::ApplyMoveOfTurn::doMove(state, list->getMove(i));
+      state.makeMove(list->getMove(i));
     }
     board->setState(state, list->getMove(n));
-    list->setSelected(index, true);
-    list->ensureCurrentVisible();
+    list->setCurrentItem(list->item(index));
   }
 }
 
 void KifuViewer::updateState()
 {
-  osl::state::SimpleState state(initialState);
+  osl::NumEffectState state(initialState);
   for (int i = 0; i <= index; i++)
   {
-    osl::ApplyMoveOfTurn::doMove(state, list->getMove(i));
+    state.makeMove(list->getMove(i));
   }
   for (int i = 0; i <= manualIndex && i < (int)manualMoves.size(); i++)
   {
-    osl::ApplyMoveOfTurn::doMove(state, manualMoves[i]);
+    state.makeMove(manualMoves[i]);
   }
   if (manualIndex >= 0 && manualIndex < (int)manualMoves.size())
     board->setState(state, manualMoves[manualIndex]);
@@ -186,11 +179,12 @@ void KifuViewer::updateState()
     board->setState(state);
   if (index == -1)
   {
-    list->setSelected(0, true);
-    list->ensureCurrentVisible();
+    list->setCurrentItem(list->item(0));
+  }
+  else
+  {
+    list->setCurrentItem(list->item(index));
   }
-  list->setSelected(index, true);
-  list->ensureCurrentVisible();
 }
 
 void KifuViewer::boardMoved(osl::Move move)
@@ -211,10 +205,20 @@ void KifuViewer::boardMoved(osl::Move move)
 void KifuViewer::updateTime()
 {
   timeView->clear();
-  new Q3ListViewItem(timeView, QString::fromUtf8("後手  "),
-		     secondToTime(consumedTime(osl::WHITE)));
-  new Q3ListViewItem(timeView, QString::fromUtf8("先手  "),
-		     secondToTime(consumedTime(osl::BLACK)));
+  QStringList headers, vheaders;
+  headers << QString::fromUtf8("消費時間");
+  vheaders << QString::fromUtf8("先手  ") << QString::fromUtf8("後手  ");
+  timeView->setHorizontalHeaderLabels(headers);
+  timeView->setVerticalHeaderLabels(vheaders);
+  QTableWidgetItem *white_time = new QTableWidgetItem(secondToTime(consumedTime(osl::WHITE)));
+  white_time->setFlags(Qt::NoItemFlags);
+  white_time->setTextAlignment(Qt::AlignRight);
+  QTableWidgetItem *black_time = new QTableWidgetItem(secondToTime(consumedTime(osl::BLACK)));
+  black_time->setFlags(Qt::NoItemFlags);
+  black_time->setTextAlignment(Qt::AlignRight);
+  timeView->setItem(0, 0, black_time);
+  timeView->setItem(1, 0, white_time);
+
   moveInfoView->clear();
   commentView->clear();
   if (0 <= index && manualIndex == -1)
@@ -224,11 +228,12 @@ void KifuViewer::updateTime()
     if (index < (int) move_info.size())
     {
       const osl::record::SearchInfo& info = move_info[index];
-      moveInfoView->insertItem
-	(new Q3ListBoxText(QString::fromUtf8("評価値 ")
-			   +QString("%1").arg(info.value)));
-      for (size_t i=0; i<info.moves.size(); ++i)
-	moveInfoView->insertItem(gpsshogi::gui::Util::moveToString(info.moves[i]));
+      new QListWidgetItem(QString::fromUtf8("評価値 ") +
+                          QString("%1").arg(info.value), moveInfoView);
+      for (size_t i = 0; i < info.moves.size(); ++i) {
+        new QListWidgetItem(gpsshogi::gui::Util::moveToString(info.moves[i]),
+                            moveInfoView);
+      }
     }
   }
 }
@@ -280,7 +285,7 @@ void KifuViewer::reloadFile()
 {
   if (! watchFileEnabled)
     return;
-  boost::mutex::scoped_lock lk(kifuFileMutex);
+  QMutexLocker lk(&kifuFileMutex);
   if (kifuFile->reloadIfChanged())
   {
     open(kifuFile.get());
@@ -365,7 +370,7 @@ public:
   {
     if (! cur_move.isNormal() || closed)
       return;
-    boost::mutex::scoped_lock lk(analysisOnlineMutex);
+    QMutexLocker lk(&analysisOnlineMutex);
     std::string euc_move = osl::record::ki2::show(cur_move, root);
     status->setText(QString::fromUtf8("探索中 %1 %2秒 NPS %3 Memory %4%")
 		    .arg(codec->toUnicode(euc_move.c_str(), euc_move.length()))
@@ -391,13 +396,13 @@ public:
       return;
     std::string euc_pv = osl::record::ki2::show(&*pv.begin(), &*pv.end(), root);
     {
-      boost::mutex::scoped_lock lk(analysisOnlineMutex);
+      QMutexLocker lk(&analysisOnlineMutex);
       pvList->setUpdatesEnabled(false);
       Q3ListViewItem *item =
 	new OnlineViewItem(pvList, depth, elapsed, node_count, 
 			   value,
 			   codec->toUnicode(euc_pv.c_str(), euc_pv.length())+additional_message,
-			   root.getTurn());
+			   root.turn());
       pvList->sort();
       pvList->setUpdatesEnabled(true);
       pvList->ensureItemVisible(item);
@@ -517,78 +522,61 @@ public:
   }	      
 };
 
-struct KifuViewer::AnalysisOnlineThread
+struct KifuViewer::AnalysisOnlineThread : public QThread, public osl::misc::Align16New
 {
-  struct Work : public osl::misc::Align16New
-  {
-    osl::NumEffectState input;
-    boost::scoped_ptr<osl::game_playing::AlphaBeta2OpenMidEndingEvalPlayer> player;
-    boost::mutex mutex;
-    volatile bool stop;
-    boost::shared_ptr<boost::thread> thread;
-    boost::shared_ptr<OnlineSearchMonitor> monitor;
-    osl::HashKey key;
-    Work() : stop(false)
-    {
-    }
-    ~Work()
-    {
-      stop = true;
-      if (thread) 
-	thread->join();
-    }
-  };
+  osl::NumEffectState input;
+  boost::scoped_ptr<osl::game_playing::AlphaBeta2OpenMidEndingEvalPlayer> player;
+  QMutex mutex;
+  volatile bool stop;
+  boost::shared_ptr<OnlineSearchMonitor> monitor;
+  osl::HashKey key;
   AnalysisOnlineThread(const osl::NumEffectState& initial) 
+    : input(initial), stop(false), key(initial)
   {
-    work.reset(new Work);
-    work->input = initial;
-    work->key = osl::HashKey(initial);
-    work->thread.reset(new boost::thread(*this));
   }
-  void operator()()
+  void run()
   {
-    while (! work->stop) {
+    while (! stop) {
       boost::scoped_ptr<osl::game_playing::GameState> state;
-      osl::HashKey key;
+      osl::HashKey last_key;
       {
-	boost::mutex::scoped_lock lk(work->mutex);
-	state.reset(new osl::game_playing::GameState(work->input));
-	work->player.reset(new osl::game_playing::AlphaBeta2OpenMidEndingEvalPlayer);
-	work->monitor.reset(new OnlineSearchMonitor(analysisOnlineDialog, work->key));
-	work->player->addMonitor(work->monitor);
-	work->player->setDepthLimit(2000, 400, 200);
-	work->player->setNodeLimit(std::numeric_limits<size_t>::max());
-	work->player->setTableLimit(std::numeric_limits<size_t>::max(), 200);
-	work->player->enableMultiPV(100);
-	work->player->setVerbose(0);
-	key = work->key;
+	QMutexLocker lk(&mutex);
+	state.reset(new osl::game_playing::GameState(input));
+	player.reset(new osl::game_playing::AlphaBeta2OpenMidEndingEvalPlayer);
+	monitor.reset(new OnlineSearchMonitor(analysisOnlineDialog, key));
+	player->addMonitor(monitor);
+	player->setDepthLimit(2000, 400, 200);
+	player->setNodeLimit(std::numeric_limits<size_t>::max());
+	player->setTableLimit(std::numeric_limits<size_t>::max(), 200);
+	player->enableMultiPV(100);
+	player->setVerbose(0);
+	last_key = key;
       }
       osl::MoveWithComment ret
-	= work->player->selectBestMove(*state, 0,0,3600);
-      while (! work->stop && key == work->key)
+	= player->selectBestMove(*state, 0,0,3600);
+      while (! stop && last_key == key)
 	boost::this_thread::sleep(boost::posix_time::milliseconds(500));
     }
   }
   void setState(const osl::NumEffectState& state)
   {
-    boost::mutex::scoped_lock lk(work->mutex);
-    if (work->key == osl::HashKey(state))
+    const osl::HashKey new_key(state);
+    QMutexLocker lk(&mutex);
+    if (key == new_key)
       return;
-    work->input = state;
-    work->key = osl::HashKey(state);
-    if (work->player && work->player->canStopSearch())
-      work->player->stopSearchNow();
+    input = state;
+    key = new_key;
+    if (player && player->canStopSearch())
+      player->stopSearchNow();
   }
   void finish()
   {
-    boost::mutex::scoped_lock lk(work->mutex);
-    work->stop = true;
-    if (work->player && work->player->canStopSearch())
-      work->player->stopSearchNow();
-    if (work->thread)
-      work->thread->join();
+    QMutexLocker lk(&mutex);
+    stop = true;
+    if (player && player->canStopSearch())
+      player->stopSearchNow();
+    wait();
   }
-  boost::shared_ptr<Work> work;
 };
 
 void KifuViewer::prepareUpdateForAnalysis(const osl::state::SimpleState &origin,
@@ -599,8 +587,8 @@ void KifuViewer::prepareUpdateForAnalysis(const osl::state::SimpleState &origin,
     return;
   osl::state::NumEffectState state(origin);
   for (size_t i = 0; i < moves.size(); i++)
-    osl::ApplyMoveOfTurn::doMove(state, moves[i]);
-  boost::mutex::scoped_lock lk(analysisOnlineMutex);
+    state.makeMove(moves[i]);
+  QMutexLocker lk(&analysisOnlineMutex);
   if (! stateForAnalysis)
     stateForAnalysis.reset(new osl::NumEffectState(state));
   else
@@ -612,7 +600,7 @@ void KifuViewer::updateStatusForAnalysis()
 {
   if (! analyzeOnlineEnabled)
     return;
-  boost::mutex::scoped_lock lk(analysisOnlineMutex);
+  QMutexLocker lk(&analysisOnlineMutex);
   if (! stateForAnalysisChanged)
     return;
   static time_t prev = 0;
@@ -629,8 +617,10 @@ void KifuViewer::updateStatusForAnalysis()
     analysisOnlineDialog->show();
   }
 
-  if (! thread)
+  if (! thread) {
     thread.reset(new AnalysisOnlineThread(*stateForAnalysis));
+    thread->start();
+  }
   else
     thread->setState(*stateForAnalysis);
   stateForAnalysisChanged = false;
@@ -651,7 +641,7 @@ void KifuViewer::setAnalyzeOnline(bool enable)
     thread.reset();
   }
   if (analysisOnlineDialog && !enable) {
-    boost::mutex::scoped_lock lk(analysisOnlineMutex);
+    QMutexLocker lk(&analysisOnlineMutex);
     AnalysisOnlineDialog *dialog = analysisOnlineDialog;
     dialog->closed = true;
     analysisOnlineDialog = 0;
@@ -705,18 +695,19 @@ void KifuViewer::open(const osl::state::SimpleState& state,
 
 void KifuViewer::open(KifuFile *file)
 {
-  boost::mutex::scoped_lock lk(kifuFileMutex);
+  QMutexLocker lk(&kifuFileMutex);
   clearState();
-  initialState = file->getInitialState();
+  initialState.copyFrom(osl::NumEffectState(file->getInitialState()));
   board->setState(initialState);
   list->setState(initialState, file->getMoves(), file->getTime(), file->getComments());
   filename = file->getFilename();
-  players->insertItem(new Q3ListBoxText(QString::fromUtf8("先手: ") +
-					file->getPlayerName(osl::BLACK)));
-  players->insertItem(new Q3ListBoxText(QString::fromUtf8("後手: ") +
-					file->getPlayerName(osl::WHITE)));
-  for (size_t i=0; i<file->getInitialComment().size(); ++i)
-    players->insertItem(new Q3ListBoxText(file->getInitialComment()[i]));
+  new QListWidgetItem(QString::fromUtf8("先手: ") +
+                      file->getPlayerName(osl::BLACK), players);
+  new QListWidgetItem(QString::fromUtf8("後手: ") +
+                      file->getPlayerName(osl::WHITE), players);
+  for (size_t i = 0; i < file->getInitialComment().size(); ++i) {
+    new QListWidgetItem(file->getInitialComment()[i], players);
+  }
   comments = file->getComments();
   move_info = file->getSearchInfo();
   if (file != kifuFile.get())
@@ -732,7 +723,7 @@ void KifuViewer::open(const osl::state::SimpleState& state,
 		      const osl::stl::vector<int>& consumed_time)
 {
   clearState();
-  initialState = state;
+  initialState.copyFrom(osl::NumEffectState(state));
   filename = "";
   board->setState(initialState);
   list->setState(initialState, board_moves, consumed_time, osl::vector<QString>());
@@ -745,7 +736,7 @@ void KifuViewer::watchFile(bool enable)
   watchFileEnabled = enable;
 }
 
-osl::state::SimpleState KifuViewer::getStateAndMovesToCurrent(osl::stl::vector<osl::Move> &m)
+osl::NumEffectState KifuViewer::getStateAndMovesToCurrent(osl::stl::vector<osl::Move> &m)
 {
   for (int i = 0; i <= index; i++)
   {
@@ -818,7 +809,7 @@ void KifuViewer::analyze()
 
 const KifuFile *KifuViewer::getKifuFile()
 {
-  boost::mutex::scoped_lock lk(kifuFileMutex);
+  QMutexLocker lk(&kifuFileMutex);
   return kifuFile.get();
 }
 
