@@ -7,7 +7,7 @@
 
 #include <qpainter.h>
 
-PiecePair::PiecePair(const osl::state::SimpleState& state,
+PiecePair::PiecePair(const osl::NumEffectState& state,
 		     QWidget *parent)
   : gpsshogi::gui::Board(state, parent)
 {
