@@ -2,14 +2,14 @@
 #define _PIECE_PAIR_DIALOG_H
 
 #include <qdialog.h>
-#include "osl/state/simpleState.h"
+#include "osl/state/numEffectState.h"
 
 class PiecePair;
 
 class PiecePairDialog : public QDialog
 {
 public:
-  PiecePairDialog(const osl::state::SimpleState &state,
+  PiecePairDialog(const osl::state::NumEffectState &state,
 		  QWidget *parent = 0, const char *name = 0);
 };
 
