@@ -1,7 +1,6 @@
 #include "viewer.h"
 #include "tabchild.h"
 #include "analysisViewer.h"
-#include "problemAnalyzer.h"
 #include "checkmateViewer.h"
 #ifdef ENABLE_NTESUKI
 #include "ntesukiViewer.h"
@@ -34,7 +33,6 @@
 #include "osl/search/simpleHashRecord.h"
 #include "osl/threatmate/mlPredictor.h"
 #include "osl/threatmate/treePredictor.h"
-#include "osl/apply_move/applyMove.h"
 #include <QFileDialog>
 #include <qlayout.h>
 #include <qtabwidget.h>
@@ -125,8 +123,8 @@ const viewer::EvalInfo Viewer::evalState()
   viewer::EvalInfo result;
   TabChild *child = (TabChild *)currentPage();
   osl::state::NumEffectState nState(child->getState());
-  if (nState.getKingPiece(osl::BLACK).isOnBoard() &&
-      nState.getKingPiece(osl::WHITE).isOnBoard())
+  if (nState.kingPiece(osl::BLACK).isOnBoard() &&
+      nState.kingPiece(osl::WHITE).isOnBoard())
   {
     osl::eval::ml::OpenMidEndingEval eval(nState);
     result.total =
@@ -149,8 +147,8 @@ void Viewer::progressOfState(osl::stl::vector<int>& out)
     return;
   }
   osl::state::NumEffectState nState(child->getState());
-  if (nState.getKingPiece(osl::BLACK).isOnBoard() &&
-      nState.getKingPiece(osl::WHITE).isOnBoard())
+  if (nState.kingPiece(osl::BLACK).isOnBoard() &&
+      nState.kingPiece(osl::WHITE).isOnBoard())
   {
     osl::progress::ml::NewProgress progress(nState);
     out.push_back(progress.progress16().value());
@@ -182,13 +180,13 @@ std::pair<double,double> Viewer::checkmateProbability()
   if ((moves.size() == 0) || (! moves.back().isNormal()))
     return std::make_pair(0.0, 0.0);
 
-  if (!state.getKingPiece(osl::BLACK).isOnBoard() ||
-      !state.getKingPiece(osl::WHITE).isOnBoard())
+  if (!state.kingPiece(osl::BLACK).isOnBoard() ||
+      !state.kingPiece(osl::WHITE).isOnBoard())
     return std::make_pair(0.0, 0.0);
 
   osl::state::NumEffectState nState(state);
   for (size_t i=0; i<moves.size(); ++i)
-    osl::ApplyMoveOfTurn::doMove(nState, moves[i]);
+    nState.makeMove(moves[i]);
   osl::threatmate::MlPredictor predictor;
   osl::threatmate::TreePredictor tree;
   return std::make_pair(predictor.probability(nState, moves.back()),
@@ -318,7 +316,7 @@ void Viewer::open()
     {
       openUsi(fileName);
     }
-    else if (selectedFile.endsWith(".ki2"))
+    else if ( selectedFile.endsWith(".ki2") || selectedFile.endsWith(".KI2") )
     {
       openKi2(fileName);
     }
@@ -430,7 +428,7 @@ void Viewer::openUrl(bool reload)
     std::string filename = http->getFilename().toStdString();
     if (http->getFilename().endsWith(".kif"))
       openKakinoki(filename);
-    else if (http->getFilename().endsWith(".ki2"))
+    else if (http->getFilename().endsWith(".ki2") || http->getFilename().endsWith(".KI2"))
       openKi2(filename);
     else if (http->getFilename().endsWith(".usi"))
       openUsi(filename);
@@ -580,27 +578,6 @@ void Viewer::analyze(bool newTab)
   }
 }
 
-void Viewer::analyzeProblems()
-{
-  ProblemAnalyzer *analyzer;
-
-  analyzer = new ProblemAnalyzer(this);
-  connect(analyzer, SIGNAL(statusChanged()),
-	  this, SIGNAL(statusChanged()));
-
-  bool success = analyzer->analyze();
-
-  if (success)
-  {
-    addTab(analyzer, "Problem Analyze");
-    showPage(analyzer);
-  }
-  else
-  {
-    delete analyzer;
-  }
-}
-
 void Viewer::checkmateSearch()
 {
   TabChild *child = (TabChild *)currentPage();
@@ -784,8 +761,7 @@ void Viewer::closeTab()
     }
     delete widget;
   }
-  else if ((widget != kifuViewer && widget->inherits("KifuViewer"))
-	   || widget->inherits("ProblemAnalyzer"))
+  else if (widget != kifuViewer && widget->inherits("KifuViewer"))
   {
     removePage(widget);
     delete widget;
